#!/bin/sh
  Version=2.0.3
# Author:       Corey Swertfager, NortonLifeLock Inc.
# Created:      07/21/2006
# Modified:     12/21/2020
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header start
# //////////////////////
#
# PROPRIETARY / CONFIDENTIAL.
# Use of this product is subject to license terms.
# Copyright (c) 2020 NortonLifeLock Inc.
# All rights reserved.
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header stop
# //////////////////////
#
# Usage:   See ShowUsage() function.
#
# Summary: See ShowHelp() function.
#
# History: 1.00 - Ported code from GetDefsDate.command.
#          1.01 - Added -c option.
#                 Removed ShowVersion function.
#          1.02 - Improved how ScriptDir is defined.
#          1.03 - Modified for case-sensitive volume compatibility.
#          2.0.0 - 11/12/2013 - Corey Swertfager:
#                  * Added support for NIS 6.
#          2.0.1 - 04/03/2020 - Corey Swertfager:
#                  * Changed Author company to NortonLifeLock Inc.
#          2.0.2 - 11/04/2020 - Corey Swertfager:
#                  * Added copyright section.
#          2.0.3 - 12/21/2020 - Corey Swertfager:
#                  * Updated copyright section.

# *** Variable Initializations ***

PATH=/bin:/sbin:/usr/bin:/usr/sbin
AbbreviatedScriptName="`basename "$0" .command`"
DefsDirName=WebFraud
DefsDir="/Library/Application Support/Symantec/$DefsDirName"
DefsDir6="/Library/Application Support/Symantec/Silo/NFM/Definitions/symcdata/webprotectiondefs"
FileToCheck=virscan1.dat
FullScriptName="`basename "$0"`"
HubDir="$DefsDir/Hub"
HubDir6Name="binhub"
LANG=""
ScriptDir="`dirname "$0"`"
[ -z "`echo "z$ScriptDir" | grep '^z/'`" ] && ScriptDir="`/bin/pwd -P`/$ScriptDir"
ShowIndividualFiles=false

# *** Function Declarations ***

GetDefsDate()
{
   # Usage:     GetDefsDate $1
   # Argument:  $1 = File from which to get date.
   # Summary:   Gets the date from a file if it is a WCID def file.
   #            Assigns date to DefsDate variable and returns 0 if
   #            a date is found, else assigns NULL to DefsDate and
   #            returns 1.
   #
   DefsDate=""
   DefsMonth=""
   # If $1 is not a file, return 1
   test -f "$1" || return 1
   # If $1 has a filename that begins with a period, return 1
   test "`basename "$1" | awk '{print substr($0,1,1)}'`" = "." && return 1
   Line=`hexdump -C -v -n 16 "$1" | head -n 1`
   # If Signature is not SYMC, return 1
   Signature=`echo "z$Line" | awk '{print $2$3$4$5}'`
   [ "z$Signature" != z53594d43 ] && return 1
   DefsMonth=`echo "z$Line" | awk '{print $10}'`
   DefsMonth=`printf %d 0x$DefsMonth`
   DefsDay=`echo "z$Line" | awk '{print $11}'`
   DefsDay=`printf %d 0x$DefsDay`
   DefsYear=`echo "z$Line" | awk '{print $12}'`
   DefsYear=`printf %d 0x$DefsYear`
   DefsYear=`expr $DefsYear + 1900`
   DefsBuild=`echo "z$Line" | awk '{print $9$8$7$6}'`
   DefsBuild=`printf %d 0x$DefsBuild`
   # If file type was not correct, return 1
   [ -z "$DefsMonth" ] && return 1
   case $DefsMonth in
      1) DefsMonthName=January
         ;;
      2) DefsMonthName=February
         ;;
      3) DefsMonthName=March
         ;;
      4) DefsMonthName=April
         ;;
      5) DefsMonthName=May
         ;;
      6) DefsMonthName=June
         ;;
      7) DefsMonthName=July
         ;;
      8) DefsMonthName=August
         ;;
      9) DefsMonthName=September
         ;;
      10) DefsMonthName=October
         ;;
      11) DefsMonthName=November
         ;;
      12) DefsMonthName=December
         ;;
   esac
   DefsItemSeq=`expr $DefsYear - 2000``printf %.2d%.2d%.3d $DefsMonth $DefsDay $DefsBuild`
   DefsDate="$DefsMonthName $DefsDay, $DefsYear - build $DefsBuild ($DefsItemSeq)"
   return 0
}

ShowDefsDateForFileOrFolder()
{
   # Usage:     ShowDefsDateForFileOrFolder $1
   # Argument:  $1 = Name of file or folder to check
   # Summary:   Shows WCID defs date for file or folder passed.
   #
   CurrentDir="`pwd`"
   DefsDate=""
   if [ -d "$1" -a $ShowIndividualFiles = false ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetDefsDate "$FileToCheck"
         # If DefsDate was assigned, break
         test -n "$DefsDate" && break
      done
      if [ -n "$DefsDate" ] ; then
         echo "Date of WCID defs in folder '$1':"
         echo "   $DefsDate"
      else
         echo "No WCID defs date was found in any file within folder: $1"
      fi
      echo
   elif [ -d "$1" -a $ShowIndividualFiles = true ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetDefsDate "$FileToCheck"
         if [ -n "$DefsDate" ] ; then
            echo "Date of WCID defs file '$1/$FileToCheck':"
            echo "   $DefsDate"
            echo
         else
            # If filename does not begin with a period (is not invisible)
            if [ "`basename "$FileToCheck" | awk '{print substr($0,1,1)}'`" != "." ] ; then
               echo "No WCID defs date was found in file: $1/$FileToCheck"
               echo
            fi
         fi
      done
   elif [ -f "$1" ] ; then
      GetDefsDate "$1"
      if [ -n "$DefsDate" ] ; then
         echo "Date of WCID defs file '$1':"
         echo "   $DefsDate"
      else
         echo "No WCID defs date was found in file: $1"
      fi
      echo
   else
      echo "No such file or folder: '$1'"
      echo
   fi
   cd "$CurrentDir"
}

ShowUsage()
{
   # Usage:     ShowUsage
   # Summary:   Displays script usage message and exits script.
   #
   echo "
Usage:   $FullScriptName [-c] [-i] [files] [folders]

Summary: Shows WCID defs date of installed WCID defs if no argument is passed,
         otherwise shows WCID defs dates of all files or folders passed.

         $FullScriptName will report the first date it finds in a given
         folder unless the -i option is passed.

         If -c is passed, then screen is not cleared. This option is useful
         when you wish to see the date of the installed defs immediately.

         If -i is passed, then dates of all individual files at the root
         level of a given folder are shown.

Examples:
         $FullScriptName
                Shows the date of the $FileToCheck file in installed defs.

         $FullScriptName -i
                Shows the date of every WCID defs file in the defs folder.

         $FullScriptName /Users/qa/Desktop/WCIDdefs\ folder
                Shows the date of the first WCID defs file it finds in
                '/Users/qa/Desktop/WCIDdefs folder' folder.

         $FullScriptName /VIRSCAN1.DAT
                Shows the date of WCID defs file VIRSCAN1.DAT, located
                at the root of the boot volume.
" | more -E
   exit 0
}

ShowUsageHelp()
{
   # Usage:     ShowUsageHelp
   # Summary:   Displays script usage message and exits script.
   #
   echo
   echo "For help, type:"
   echo
   echo "   $FullScriptName -h"
   echo
   exit 0
}

# *** Beginning of Commands to Execute ***

# If no arguments were passed to script
if [ $# -eq 0 ] ; then
   # Run script as if it was double-clicked in Finder so that
   # screen will be cleared and quit message will be displayed.
   RunScriptAsStandAlone=true
else
   # Run script in command line mode so that
   # screen won't be cleared and quit message won't be displayed.
   RunScriptAsStandAlone=false
fi
if [ $RunScriptAsStandAlone = true ] ; then
   clear
fi
if [ "'$1" = "'-h" -o "'$1" = "'-H" ] ; then
   ShowUsage
fi
if [ "'$1" = "'-V" ] ; then
   echo $Version
   exit 0
fi
# If not root user,
if [ "`whoami`" != "root" ] ; then
   # Run this script again as root
   sudo -p "Please enter your admin password: " "$0" "$@"
   if [ ! $? -eq 0 ] ; then   #    If unable to authenticate,
      echo "You entered an invalid password or you are not an admin user. Script aborted."
      exit 1
   fi
   exit 0                     #    Exit so script doesn't run again
fi
while [ "$1" ] ; do
   if [ "z$1" = z-i ] ; then
      ShowIndividualFiles=true
      shift
   elif [ "z$1" = z-c ] ; then
      shift
   else
      break
   fi
done
if [ ! -d "$1" -a ! -f "$1" -a -n "$1" ] ; then
   echo
   echo "Invalid option, file, or folder: '$1'"
   ShowUsageHelp
fi
NumberOfArgumentsPassedToScript=$#
NumberOfArgumentsLeft=$#
echo
while [ $NumberOfArgumentsLeft != 0 ] ; do
   ShowDefsDateForFileOrFolder "$1"
   shift
   NumberOfArgumentsLeft=`expr $NumberOfArgumentsLeft - 1`
done
test $NumberOfArgumentsPassedToScript -gt 0 && exit 0
cd "$DefsDir" 2>/dev/null
if [ $? = 0 ] ; then
   cd ..
   ShowDefsDateForFileOrFolder "$DefsDirName"
   ShowDefsDateForFileOrFolder "$DefsDirName/Hub"
elif [ -d "$DefsDir6" ] ; then
   cd "$DefsDir6"
   IFS='
'
   for Folder in `ls -d 2* "$HubDir6Name" 2>/dev/null` ; do
      ShowDefsDateForFileOrFolder "$Folder"
   done
else
   echo "Could not find any installed anti-phishing (WCID) definitions."
   echo
   exit 2
fi
exit 0

# *** End of Commands to Execute ***
