#!/bin/sh
  Version=3.0.22
# Author:       Corey Swertfager, NortonLifeLock Inc.
# Created:      03/19/2003
# Modified:     12/21/2020
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header start
# //////////////////////
#
# PROPRIETARY / CONFIDENTIAL.
# Use of this product is subject to license terms.
# Copyright (c) 2020 NortonLifeLock Inc.
# All rights reserved.
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header stop
# //////////////////////
#
# Usage:        See ShowHelp() function.
#
# Summary:      See ShowHelp() function.
#
# History: 1.00 - Created.
#          1.01 - Updated Path definition and help.
#          1.02 - Updated ShowSMacVersion function (formally called
#                 ShowSymMacCryptVersion) to handle manually built SMac.
#          1.03 - Fixed version reporting for SMac server.
#                 Renamed ShowSMacVersion function to ShowSACMVersion.
#          1.04 - Added check for Symantec Scheduler.app.
#                 Added check within Symantec Solutions.
#                 Revamped output.
#          1.05 - Added -n option.
#          1.06 - Removed extra line output when -n option is given.
#          1.07 - Added check for Norton Confidential.app and for
#                 Symantec Uninstaller.app.
#                 Revised how versions and build numbers are retrieved
#                 to account for different info.plist layouts.
#          1.08 - Adjusted AssignVolume function.
#          1.09 - Modified for OS 10.5 compatibility.
#          1.10 - Added build or version to unknown output.
#          1.11 - Added quotes when displaying non-/ volume names.
#          1.12 - Added Norton Firewall.app.
#          1.13 - Adjusted help.
#          2.00 - Added -a option and ability to pass multiple volumes.
#          2.01 - Converted -n comparison in ShowFullFilePath function.
#          2.02 - Now looks for products matching "Norton "* and "Symantec "*.
#          2.03 - Added Norton Safety Minder.
#          2.04 - Added PATH definition.
#          2.05 - Norton Safety Minder version is now determined via
#                 SymQuickMenu/Norton Safety Minder QM.plugIn.
#          2.06 - Replaced ShowFullFilePath function with version 1.0.1.
#          2.07 - Added Norton DNS.
#                 Added ShowLooseInApplicationsFiles function.
#                 Now shows Symantec/Norton files/apps that are loose in
#                 /Applications folder even when there is no Norton Solutions
#                 or Symantec Solutions folder in /Applications folder.
#          2.08 - Added folder option.
#          3.0.0 - Merged ShowRegistryInfo program into this program.
#                  Added options: -b, -c, -s, -X, -x.
#                  Added folder option.
#          3.0.1 - Fixed incorrect volume names appearing when -c was passed
#                  when -a and/or multiple volumes were passed.
#                  Removed signature from registry output.
#          3.0.2 - Modified for case-sensitive volume compatibility.
#                  Replaced ShowFullFilePath function with version 1.0.2.
#          3.0.3 - Added display of LiveUpdate 6 registry plists.
#                  Added -p option to display full plist contents.
#                  Changed the option to show info on all mounted volumes
#                  from -a to -A.
#                  Added -a option to show registry info after version info.
#                  Made the -b option (show registry info before version
#                  info) the default.
#          3.0.4 - Added -i option to show only application info and
#                  -r option to show only registry info.
#          3.0.5 - Added display of stage 2 build info.
#          3.0.6 - Removed display of extra blank lines in stage 2 build info.
#          3.0.7 - Added -e option to limit product output using pattern.
#                  Added -m option to display messages for missing apps/registries.
#                  Added -q option to suppress output of missing apps/registries
#                  The is now the default behavior.
#                  Changed "Rockefeller Center" to "Symantec Administration Console".
#          3.0.8 - 08/23/2011:
#                  Adjusted error when no version or host info is available.
#                  Made -e optional when specifying a pattern.
#          3.0.9 - 09/12/2011:
#                  Added WCID Engine info.
#          3.0.10 - 01/24/2012:
#                   Added display of LiveUpdate 6.1 ActiveRegistry files.
#          3.0.11 - 05/22/2012:
#                  * Added iAntiVirus.
#                  * Modified check of InstallerPath to use -e instead of -d.
#          3.0.12 - 07/06/2012 - Corey Swertfager:
#                   Modified grep calls with regular expressions that contain pattern "$\|"
#                   to instead use extended regular expression "$|" for OS 10.8 compatibility.
#          3.0.13 - 10/16/2012 - Corey Swertfager:
#                   Added Norton Zone.
#          3.0.14 - 08/16/2013 - Corey Swertfager:
#                   * Added display of NIS 6 LUX component registry entries
#                     if ViewDatastore.command exists.
#                   * Removed square brackets [] from registry header.
#          3.0.15 - 01/16/2014 - Corey Swertfager:
#                   * Added SmallScanner.
#                   * Suppressed output of stage 2 items that do not contain Name key in
#                     their plists.
#                   * Improved the way CFBundleGetInfoString is found and parsed.
#          3.0.16 - 04/18/2014 - Corey Swertfager:
#                   * Added Norton Online Backup in:
#                     /Library/Application Support/Norton Online Backup/Norton Online Backup.app
#          3.0.17 - 08/11/2014 - Corey Swertfager:
#                   * Adjusted path to WCIDEngine.bundle when checking non-boot volume.
#          3.0.18 - 03/22/2019 - Corey Swertfager:
#                   * Added Norton Secure VPN.
#          3.0.19 - 11/13/2019 - Corey Swertfager:
#                   * Now shows the value of the key CFBundleShortVersionString
#                     when found in plists.
#          3.0.20 - 04/03/2020 - Corey Swertfager:
#                  * Added NortonLifeLock Inc. to Author.
#          3.0.21 - 11/04/2020 - Corey Swertfager:
#                   * Added copyright section.
#          3.0.22 - 12/21/2020 - Corey Swertfager:
#                   * Updated copyright section.

# *** Variable Initializations ***

PATH=/bin:/sbin:/usr/bin:/usr/sbin
LANG=""
ActiveRegistryDirLiveUpdate6="/Library/Application Support/Symantec/LiveUpdate/ActiveRegistry"
NewRegistryDirLiveUpdate6="/Library/Application Support/Symantec/LiveUpdate/NewRegistry"
NortonAppDirName="Norton Solutions"
RegistryDirLiveUpdate5="/Library/Application Support/Norton Solutions Support/LiveUpdate/Registry"
SymantecAppDirName="Symantec Solutions"
VPNAppName="Norton Secure VPN"

# *** Function Declarations ***

ExitScript()
{
   # Usage:     ExitScript [-b] [-e] [exit_number [error_string]]
   # Summary:   Checks to see if ShowQuitMessage and RunScriptAsStandAlone
   #            variables are set to true. If so, a message is displayed;
   #            otherwise, no message is displayed. The script is then
   #            exited and passes exit_number to exit command. If no
   #            exit_number is passed, then 0 is passed to exit command. If
   #            a non-integer is passed as exit_number, 255 is passed to
   #            exit command. If error_string is passed, it is printed to
   #            to standard out before exiting and is padded by blank lines 
   #            if error_string is not "". Pass -b before exit_number to
   #            suppress beginning padding line, -e to suppress ending
   #            padding line, both to suppress both.
   #
   local PadBeginning=true
   local PadEnd=true
   while [ "$1" ] ; do
      case "$1" in
         -b)
            PadBeginning=false
            ;;
         -e)
            PadEnd=false
            ;;
         *)
            break
            ;;
      esac
      shift
   done
   if [ $# -gt 1 ] ; then
      if [ -z "$2" ] ; then
         PadBeginning=false
         PadEnd=false
      fi
      $PadBeginning && echo 
      printf "%s\n" "$2"
      $PadEnd && echo
   fi
   if [ "z$ShowQuitMessage" = ztrue -a "z$RunScriptAsStandAlone" = ztrue ] ; then
      [ $# -lt 2 -o \( $PadEnd = false -a -n "$2" \) ] && echo
      echo "NOTE: If you double-clicked this program, quit Terminal application now."
      [ $PadEnd = true -o -z "$2" ] && echo
   fi
   [ -z "$1" ] && exit 0
   [ -z "`expr "$1" / 1 2>/dev/null`" ] && exit 255 || exit $1
}

GetVersion()
{
   # Usage:     GetVersion String
   # Argument:  String = Partial string from CFBundleGetInfoString.
   #                     Pass String without quotes to separate portions.
   # Summary:   Echoes the version portion of String.
   #
   StringToShow=""
   while [ "$1" ] ; do
      StringToShow="$1"
      shift
   done
   echo "$StringToShow"
}

ListInfoPlist()
{
   # Usage:     ListInfoPlist [-n] $1 $2 [$3]
   # Arguments: $1 = Full path of directory that contains file $2.
   #            $2 = Name of file.
   #            $3 = Name to display.
   # Summary:   Displays contents of file's Contents/info.plist
   #            If -n is passed as first argument, output is not indented.
   #
   if [ "z$1" = z-n ] ; then
      shift
      IndentToAdd=""
   else
      IndentToAdd="   "
   fi
   # If the file has .pkg extension or it is a symbolic link, ignore it
   if [ "z`echo "$1/$2" | grep "\.pkg"`" != "z" -o -L "$1/$2" ] ; then
      return
   fi
   FileWithNoExtension=`basename "$2" .app`
   if [ "$3" ] ; then
      FileNameToShow="$3"
   elif [ "z$FileWithNoExtension" = "zNorton Disk Doctor" ] ; then
      FileNameToShow="Norton Utilities"
   else
      FileNameToShow="$FileWithNoExtension"
   fi
   ThePlist=`ls -d "$1/$2/Contents/"[Ii]"nfo.plist" 2>/dev/null`
   # If file does not contain an info.plist, show if not a package
   if [ ! -f "$ThePlist" ] ; then
      if [ -f "$1/$FileWithNoExtension" ] ; then
         echo "$IndentToAdd$FileNameToShow (not a bundle - perhaps it's a pre-Hangar file)"
         echo
      fi 
      return
   fi
   printf "$IndentToAdd$FileNameToShow "
   CFBundleShortVersionString=`defaults read "$ThePlist" CFBundleShortVersionString 2>/dev/null`
   if [ -n "$CFBundleShortVersionString" ] ; then
      CFBundleVersion=`defaults read "$ThePlist" CFBundleVersion 2>/dev/null`
      BuildVersion=`defaults read "$ThePlist" BuildVersion 2>/dev/null`
      if [ "z$CFBundleVersion" = "z$BuildVersion" ] ; then
         BuildVersion=""
      fi
      if [ -n "$CFBundleVersion" ] ; then
         ExtraVersionInfoToShow=" ($CFBundleVersion`[ -n "$BuildVersion" ] && echo " - $BuildVersion"`)"
      else
         ExtraVersionInfoToShow=""
      fi
      echo "$CFBundleShortVersionString$ExtraVersionInfoToShow"
      echo
      return
   fi
   CFBundleGetInfoString=`grep -A1 '>CFBundleGetInfoString<' "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
   # Set up CFBundleGetInfoStringAdjusted based on presence of paretheses
   if [ "$CFBundleGetInfoString" ] ; then
      CFBundleGetInfoStringAdjusted=`printf "%s" "$CFBundleGetInfoString" | grep "(" | awk '{print $1" "$2}' | tr -d ,`
   else
      # Use the line with parentheses to set CFBundleGetInfoStringAdjusted
      CFBundleGetInfoStringAdjusted=`grep "(" "$ThePlist" | awk '{print $1" "$2}' | tr -d "<string>,"`
   fi
#   [ "`echo "$CFBundleGetInfoStringAdjusted" | awk '{print $2}' | tr -d "[:alpha:]"`" = "()" ] && printf "(pre-Hangar) "
#   [ -z "$CFBundleGetInfoStringAdjusted" ] && printf "(pre-Hangar) "
   # If Norton Zone
   if [ "z$FileNameToShow" = "zNorton Zone" ] ; then
      ProductVersion=`grep -A 1 CFBundleShortVersionString "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      if [ "$ProductVersion" ] ; then
         # If there is a build field
         if [ "`echo "$ProductVersion" | awk -F . '{print $4}'`" ] ; then
            CFBundleGetInfoStringAdjusted="`echo "$ProductVersion" | awk -F . '{print $1"."$2"."$3" ("$4")"}'`"
         else
            CFBundleGetInfoStringAdjusted="`echo "$ProductVersion" | awk -F . '{print $1"."$2"."$3" (build unknown)"}'`"
         fi
      else
         CFBundleGetInfoStringAdjusted="(version unknown)"
      fi
      echo "$CFBundleGetInfoStringAdjusted"
      echo
      return
   # Else if there were no lines with parentheses
   elif [ -z "$CFBundleGetInfoStringAdjusted" ] ; then
      # Grab everything before first comma
      if [ "$CFBundleGetInfoString" ] ; then
         CFBundleGetInfoStringAdjusted=`printf "%s" "$CFBundleGetInfoString" | awk -F ',' '{print $1}'`
      else
         # Use the line with "Symantec Corp" to set CFBundleGetInfoStringAdjusted
         CFBundleGetInfoStringAdjusted=`grep "Symantec Corp" "$ThePlist" | awk -F '>' '{print $2}' | awk -F ',' '{print $1}'`
      fi
      CFBundleGetInfoStringAdjusted1=`echo "$CFBundleGetInfoStringAdjusted" | awk '{print $1}'`
      CFBundleGetInfoStringAdjusted2=`echo "$CFBundleGetInfoStringAdjusted" | awk '{print $2}'`
      CFBundleGetInfoStringAdjusted3=`echo "$CFBundleGetInfoStringAdjusted" | awk '{print $3}'`
      CFBundleGetInfoStringAdjusted4=`echo "$CFBundleGetInfoStringAdjusted" | awk '{print $4}'`
      CFBundleGetInfoStringAdjusted5=`echo "$CFBundleGetInfoStringAdjusted" | awk '{print $5}'`
      CFBundleGetInfoStringAdjusted=`GetVersion "$CFBundleGetInfoStringAdjusted1" "$CFBundleGetInfoStringAdjusted2" "$CFBundleGetInfoStringAdjusted3" "$CFBundleGetInfoStringAdjusted4" "$CFBundleGetInfoStringAdjusted5"`
      # If CFBundleGetInfoStringAdjusted is not null and contains no letters
      if [ "$CFBundleGetInfoStringAdjusted" -a -z "`echo "$CFBundleGetInfoStringAdjusted" | grep '[a-zA-Z]'`" ] ; then
         BuildVersion=`grep -A 1 BuildVersion "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
         [ "$BuildVersion" ] && CFBundleGetInfoStringAdjusted="$CFBundleGetInfoStringAdjusted ($BuildVersion)"
      fi
   fi
   if [ -z "$CFBundleGetInfoStringAdjusted" ] ; then
      BuildVersion=`grep -A 1 BuildVersion "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      ProductVersion=`grep -A 1 CFBundleShortVersionString "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      if [ "$ProductVersion" ] ; then
         [ -z "$BuildVersion" ] && BuildVersion=`grep -A 1 CFBundleVersion "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      else
         ProductVersion=`grep -A 1 CFBundleVersion "$ThePlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      fi
      if [ "$ProductVersion" ] ; then
         [ -z "$BuildVersion" ] && BuildVersion="build unknown"
         CFBundleGetInfoStringAdjusted="$ProductVersion ($BuildVersion)"
      fi
   fi
   [ -z "$CFBundleGetInfoStringAdjusted" ] && CFBundleGetInfoStringAdjusted="(version unknown)"
   echo "$CFBundleGetInfoStringAdjusted"
   echo
}

ProcessArguments()
{
   # Usage:     ProcessArguments "$@"
   # Summary:   Processes arguments passed to script.
   #
   NumberOfProcessArgumentsLeft=$#
   DoShowAppInfo=true
   DoShowFullPlist=false
   DoShowMessageIfNotInstalled=true
   DoShowRegistryInfo=true
   RegistryInfoBefore=true
   LimitOutputPattern=""
   ListOfPaths=""
   LU5HeaderAddOn1=""
   LU5HeaderAddOn2=""
   LU6HeaderAddOn1=""
   LU6HeaderAddOn2=""
   Option=""
   ViewDatastoreOption=-R
   Verbose=true
   while [ $NumberOfProcessArgumentsLeft != 0 ] ; do
      case "$1" in
         -A)
            ListOfPaths="$ListOfPaths
`ls -d /Volumes/* 2>/dev/null`"
            ;;
         -a)
            RegistryInfoBefore=false
            ;;
         -b)
            RegistryInfoBefore=true
            ;;
         -c)
            LU5HeaderAddOn1=""
            LU5HeaderAddOn2="   ProductName, Version, Language, ItemSeqName (Sequence Type), ItemSeqData (Sequence Number)"
            LU6HeaderAddOn1=""
            LU6HeaderAddOn2="$LU5HeaderAddOn2"
            Option="$1"
            Verbose=false
            ViewDatastoreOption=-r
            DoShowFullPlist=false
            ;;
         -e)
            [ -z "$2" ] && ShowUsage 2 "ERROR: No pattern was passed after -e."
            LimitOutputPattern="$2"
            shift
            NumberOfProcessArgumentsLeft=`expr $NumberOfProcessArgumentsLeft - 1`
            ;;
         -H)
            ShowUsage 0
            ;;
         -h)
            ShowHelp 0
            ;;
         -i)
            DoShowAppInfo=true
            DoShowRegistryInfo=false
            ;;
         -p)
            LU6HeaderAddOn1=" (full plist contents)"
            LU6HeaderAddOn2=""
            DoShowFullPlist=true
            ;;
         -q)
            DoShowMessageIfNotInstalled=false
            ;;
         -r)
            DoShowAppInfo=false
            DoShowRegistryInfo=true
            ;;
         -s)
            LU5HeaderAddOn1=""
            LU5HeaderAddOn2="   Version               ItemSeqData (Sequence Number)"
            LU6HeaderAddOn1=""
            LU6HeaderAddOn2="   Version -> ItemSeqData (Sequence Number)"
            Option="$1"
            Verbose=false
            ViewDatastoreOption=-r
            DoShowFullPlist=false
            ;;
         -V)
            echo $Version
            exit 0
            ;;
         -X)
            LU5HeaderAddOn1=" (full hexdump)"
            LU5HeaderAddOn2=""
            Option="$1"
            ;;
         -x)
            LU5HeaderAddOn1=" (ASCII hexdump)"
            LU5HeaderAddOn2=""
            Option="$1"
            ;;
         "")
               ListOfPaths="$ListOfPaths
/"
            ;;
         *)
            if [ -d "$1" ] ; then
               ListOfPaths="$ListOfPaths
`ShowFullFilePath -P "$1"`"
            elif [ -z "$LimitOutputPattern" ] ; then
               LimitOutputPattern="$1"
            else
               ShowUsage 2 "ERROR: Not a valid option, folder, or volume:
       \"$1\""
            fi
            ;;
      esac
      shift
      NumberOfProcessArgumentsLeft=`expr $NumberOfProcessArgumentsLeft - 1`
   done
   if [ -z "$ListOfPaths" ] ; then
      ListOfPaths=/
   fi
   if [ $DoShowAppInfo = false ] ; then
      RegistryInfoBefore=true
   elif [ $DoShowRegistryInfo = false ] ; then
      RegistryInfoBefore=false
   fi
}

ShowFullFilePath()
{
   # Usage:     ShowFullFilePath [-a] [-P | -L] [-e] Path [[-e] Path]
   # Version:   1.0.2
   # Summary:   Prints the full path starting at / of Path if Path exists
   #            and Path is accessible by the user calling this function.
   #            Run this function as root to ensure full path displaying.
   #            If there is more than one existing file that matches the
   #            name, then only the first path that the shell matches is
   #            printed unless -a or more than one path is specified.
   #            You can specify wild card characters ? and * and other
   #            argument operators in the Path (e.g., "../*", "a?.txt",
   #            "[ab]*").
   # Options:   -a   Show all matching paths, sorted alphanumerically. If
   #                 -P is not passed, the same file may be shown multiple
   #                 times if there is more than one matching link that
   #                 points to it.
   #            -e <Path>
   #                 Treat argument after -e as a path. Use -e to treat
   #                 -a, -e, -L, or -P as a path.
   #            -L   Show logical path, even if a file pointed to by a link
   #                 doesn't exist. This is the default.
   #            -P   Show physical path. If a link points to a file that
   #                 does not exist, the path won't be shown.
   # History: 1.0.1 - Added -e option and ability to pass multiple paths.
   #                  Arguments can now be passed in any order.
   #                  Fixed error that could occur when resolving links
   #                  with long paths.
   #          1.0.2 - Modified for case-sensitive volume compatibility.
   #                  Made temporary file names more distinctive.
   #
   local SFFPArgCount=$#
   local SFFPCurrentDir
   local SFFPCurrentDirTranslated
   local SFFPEachLine
   local SFFPEachPath
   local SFFPFile
   local SFFPLDir
   local SFFPLLinkLS
   local SFFPLLinkPath
   local SFFPLPath
   local SFFPPathOption=-L
   local SFFPSaveIFS="$IFS"
   local SFFPShowAll=false
   local SFFPTempBase=/private/tmp/ShowFullFilePath-`/usr/bin/basename "$0"`-`/bin/date +"%Y%m%d%H%M%S"`
   local SFFPTempFile="$SFFPTempBase.tmp"
   local SFFPTempFile2="$SFFPTempBase-2.tmp"
   /bin/rm -f "$SFFPTempFile" 2>/dev/null
   while [ $SFFPArgCount != 0 ] ; do
      case "$1" in
         -a)
            SFFPShowAll=true
            ;;
         -L|-P)
            SFFPPathOption="$1"
            ;;
         *)
            [ "z$1" = z-e ] && shift
            if [ "$1" ] ; then
               [ -s "$SFFPTempFile" ] && SFFPShowAll=true
               /usr/bin/printf "%s\n" "$1" >>"$SFFPTempFile"
            fi
            ;;
      esac
      shift
      let SFFPArgCount=$SFFPArgCount-1
   done
   [ ! -s "$SFFPTempFile" ] && return
   SFFPCurrentDir=`/bin/pwd`
   SFFPCurrentDirTranslated=`/bin/pwd $SFFPPathOption 2>/dev/null`
   if [ ! -d "$SFFPCurrentDirTranslated" ] ; then
      /bin/rm -f "$SFFPTempFile" 2>/dev/null
      return
   fi
   cd "$SFFPCurrentDirTranslated" 2>/dev/null
   if [ $? != 0 ] ; then
      /bin/rm -f "$SFFPTempFile" 2>/dev/null
      return
   fi
   /usr/bin/printf "" >"$SFFPTempFile2"
   IFS='
'
   for SFFPEachLine in `/bin/cat "$SFFPTempFile" 2>/dev/null` ; do
      cd "$SFFPCurrentDirTranslated" 2>/dev/null
      [ $? != 0 ] && break
      if [ "z$SFFPPathOption" = z-P ] ; then
         SFFPLPath="$SFFPEachLine"
         while [ -L "$SFFPLPath" ] ; do
            [ ! -e "$SFFPLPath" ] && break
            cd "`/usr/bin/dirname "$SFFPLPath" 2>/dev/null`" 2>/dev/null
            [ $? != 0 ] && break
            SFFPLDir=`/bin/pwd -P 2>/dev/null`
            [ ! -d "$SFFPLDir" ] && break
            SFFPLLinkLS=`/bin/ls -ld "$SFFPLPath" 2>/dev/null`
            [ -z "$SFFPLLinkLS" ] && break
            # If link or link target contains " -> " in its name
            if [ "`echo "z$SFFPLLinkLS" | grep ' -> .* -> '`" ] ; then
               SFFPLLinkPath=`/usr/bin/printf "%s" "$SFFPLLinkLS" | /usr/bin/awk -v THESTR="$SFFPLPath -> " '{ match($0,THESTR) ; print substr($0,RSTART+RLENGTH)}'`
            else
               SFFPLLinkPath=`echo "$SFFPLLinkLS" | awk -F " -> " '{print $2}'`
            fi
            # If link target begins with /
            if [ "`/usr/bin/printf "%s" "$SFFPLLinkPath" | grep '^/'`" ] ; then
               SFFPLPath="$SFFPLLinkPath"
            else
               SFFPLPath="$SFFPLDir/$SFFPLLinkPath"
            fi
            [ "`/usr/bin/printf "%s" "$SFFPLPath" | grep '^//'`" ] && SFFPLPath=`echo "$SFFPLPath" | /usr/bin/awk '{print substr($0,2)}'`
         done
         cd "$SFFPCurrentDirTranslated" 2>/dev/null
         [ $? != 0 ] && break
         if [ ! -e "$SFFPLPath" ] ; then
            $SFFPShowAll && continue || break
         fi
         SFFPEachPath="$SFFPLPath"
      else
         SFFPEachPath="$SFFPEachLine"
      fi
      if [ -d "$SFFPEachPath" ] ; then
         cd "$SFFPEachPath" 2>/dev/null
         if [ $? != 0 ] ; then
            $SFFPShowAll && continue || break
         fi
         SFFPFile=""
      elif [ -d "`/usr/bin/dirname "$SFFPEachPath" 2>/dev/null`" ] ; then
         cd "`/usr/bin/dirname "$SFFPEachPath" 2>/dev/null`" 2>/dev/null
         if [ $? != 0 ] ; then
            $SFFPShowAll && continue || break
         fi
         SFFPFile=`basename "$SFFPEachPath" 2>/dev/null`
         [ "z$SFFPFile" = z/ -o "z$SFFPFile" = z. -o "z$SFFPFile" = z.. ] && SFFPFile=""
      elif $SFFPShowAll ; then
         continue
      else
         break
      fi
      SFFPDir=`/bin/pwd $SFFPPathOption 2>/dev/null`
      if [ ! -d "$SFFPDir" ] ; then
         $SFFPShowAll && continue || break
      fi
      SFFPPath="$SFFPDir`[ "z$SFFPFile" != z -a "z$SFFPDir" != z/ -a "z$SFFPDir" != z// ] && echo /`$SFFPFile"
      if [ ! -e "$SFFPPath" -a ! -L "$SFFPPath" ] ; then
         $SFFPShowAll && continue || break
      fi
      [ "`echo "$SFFPPath" | grep '^//'`" ] && SFFPPath=`echo "$SFFPPath" | /usr/bin/awk '{print substr($0,2)}'`
      echo "$SFFPPath" >>"$SFFPTempFile2"
      # If neither option -a nor more than one path was passed, don't show any more names
      ! $SFFPShowAll && break
   done
   IFS=$SFFPSaveIFS
   [ -s "$SFFPTempFile2" ] && /usr/bin/sort -f "$SFFPTempFile2" | /usr/bin/uniq
   /bin/rm -f "$SFFPTempFile" "$SFFPTempFile2" 2>/dev/null
   cd "$SFFPCurrentDir" 2>/dev/null
}

ShowHelp()
{
   # Usage:     ShowHelp [$1]
   # Argument:  $1 = Exit code.
   # Summary:   Displays script usage and help then exits script.
   #            If a number is passed to $1, then script exits with
   #            that number; else, script is not exited.
   #
   ShowHelpCurrentDir=`pwd`
   ShowHelpTempFolder="/private/tmp/$FullScriptName-ShowHelp-`date +"%Y%m%d%H%M%S"`"
   mkdir "$ShowHelpTempFolder" 2>/dev/null
   cd "$ShowHelpTempFolder" 2>/dev/null
   ShowHelpTempFile="....."
   echo "
Usage: $FullScriptName [volume | folder] [-a] [-b] [-c | -s | -X | -x] [-p] [-q] [-i | -r] [[-e] <pattern>]

       Shows version info for installed Symantec products based on their
       Contents/Info.plist. Screen is cleared before info is shown if
       nothing is passed to $FullScriptName.

       If Symantec Administration Console (SACM) is installed, version is
       obtained from AddGroups command. The number of installed SACM man
       pages is also shown.

       Also displays contents of LiveUpdate 5 and earlier registry files
       created by LURegistryTool in:

       $RegistryDirLiveUpdate5

       and displays contents of LiveUpdate 6 registry plists in:

       $ActiveRegistryDirLiveUpdate6
       $NewRegistryDirLiveUpdate6

       If ViewDatastore.command is installed, NIS 6 LUX component registry
       entries are also shown.

       Info is reported for each volume and folder passed. If no volumes
       or folders are passed, info is reported for current boot volume.

Options:

   -A    Display info for all mounted volumes.
   -a    Display registry info after application info.
   -b    Display registry info before application info (this is the default).
   -c    Display fields of each registry file compacted on one line.
   -e <pattern>
         Only show apps/registries that match pattern. Letter case is ignored.
            Example: $FullScriptName -e 'LiveUpdate\|LUlu'
         The -e is optional unless you want to make sure pattern is not
         treated as another argument or folder by $FullScriptName.
            Example: $FullScriptName .
                     Current working directory would be searched.
            Example: $FullScriptName -e .
                     Pattern is set to \".\", which will match any character
                     and thus all apps/registries would be shown.
   -i    Display only application info.
   -m    Show messages when apps or registries are missing.
   -p    Display full LiveUpdate 6 and later plist contents.
   -q    Do not show message when apps or registries are not installed;
         this is the default behavior as of version 3.0.7.
   -r    Display only registry info.
   -s    Display registry fields succinctly; shows only Version and ItemSeqData.
         NIS 6 registry entries will be displayed in compacted form (-c option).
   -X    Display entire hexdump of LiveUpdate 5 registry files.
   -x    Display ASCII of entire hexdump of LiveUpdate 5 registry files.
" > "$ShowHelpTempFile"
   if [ `grep -c "" "$ShowHelpTempFile"` -gt 24 ] ; then
      clear
      more -E "$ShowHelpTempFile"
   else
      cat "$ShowHelpTempFile"
   fi
   cd "$ShowHelpCurrentDir" 2>/dev/null
   rm -rf "$ShowHelpTempFolder" 2>/dev/null
   [ "$1" ] && exit $1
}

ShowInstalledApplicationInfo()
{
   if [ "$Path" = / ] ; then
      PathToUseAsPrefix=""
   else
      PathToUseAsPrefix="$Path"
   fi
   NortonAppDir="$PathToUseAsPrefix/Applications/$NortonAppDirName"
   SymantecAppDir="$PathToUseAsPrefix/Applications/$SymantecAppDirName"
   NortonOnlineBackup="/Library/Application Support/Norton Online Backup/Norton Online Backup.app"
   if [ ! -d "$NortonAppDir" -a ! -d "$SymantecAppDir" ] ; then
      ShowLooseInApplicationsFiles
      if $DoShowMessageIfNotInstalled ; then
         echo "No $NortonAppDirName or $SymantecAppDirName found$VolumeText."
         echo
      fi
   else
      ShowLooseInApplicationsFiles
      AppInfoWasShown=false
      PathsOfApps=`ls -d "$NortonAppDir/LiveUpdate"* "$NortonAppDir/Norton "* "$NortonAppDir/Symantec "* "$NortonAppDir/Norton Utilities/Norton Disk Doctor"* 2>/dev/null | sort -f | uniq | grep -ie "$NortonAppDir/$LimitOutputPattern"`
      if [ -d "$NortonAppDir" -a ! -L "$NortonAppDir" -a -n "$PathsOfApps" ] ; then
         echo "Products installed in $NortonAppDirName$VolumeText:"
         echo
         for EachPath in $PathsOfApps ; do
            ListInfoPlist "`dirname "$EachPath"`" "`basename "$EachPath"`"
         done
         AppInfoWasShown=true
      fi
      PathsOfApps=`ls -d "$SymantecAppDir/LiveUpdate"* "$SymantecAppDir/Norton "* "$SymantecAppDir/SmallScanner.app" "$SymantecAppDir/Symantec "* "$SymantecAppDir/Norton Utilities/Norton Disk Doctor"* 2>/dev/null | sort -f | uniq | grep -ie "$SymantecAppDir/$LimitOutputPattern"`
      if [ -d "$SymantecAppDir" -a ! -L "$SymantecAppDir" -a -n "$PathsOfApps" ] ; then
         echo "Products installed in $SymantecAppDirName$VolumeText:"
         echo
         for EachPath in $PathsOfApps ; do
            ListInfoPlist "`dirname "$EachPath"`" "`basename "$EachPath"`"
         done
         AppInfoWasShown=true
      fi
      if ! $AppInfoWasShown ; then
         if $DoShowMessageIfNotInstalled ; then
            echo "No apps found in $NortonAppDirName or $SymantecAppDirName$VolumeText"
            [ -n "$LimitOutputPattern" ] && echo "that begin with pattern \"$LimitOutputPattern\""
            echo
         fi
      fi
   fi
   if [ -d "$NortonOnlineBackup" ] ; then
      NortonOnlineBackupDir=`dirname "$NortonOnlineBackup"`
      echo "In \"$NortonOnlineBackupDir\"$VolumeText:"
      echo
      ListInfoPlist "$NortonOnlineBackupDir" "`basename "$NortonOnlineBackup"`"
      AppInfoWasShown=true
   fi
   if [ -d "$PathToUseAsPrefix/Library/Application Support/Symantec/SymQuickMenu/Norton Safety Minder QM.plugIn" -a \
        -n "`echo "Norton Safety Minder QM.plugIn" | grep -ie "^$LimitOutputPattern"`" ] ; then
      echo "Installed in SymQuickMenu folder:"
      echo
      ListInfoPlist "$PathToUseAsPrefix/Library/Application Support/Symantec/SymQuickMenu" "Norton Safety Minder QM.plugIn" "Norton Safety Minder"
   fi
   # Show stage 2 build and other components' versions
   iAntiVirusPlist="$PathToUseAsPrefix/Applications/iAntivirus.app/Contents/Info.plist"
   iAntiVirusPlistStrings="$PathToUseAsPrefix/Applications/iAntivirus.app/Contents/Resources/English.lproj/InfoPlist.strings"
   SymantecSupportDir="$PathToUseAsPrefix/Library/Application Support/Symantec"
   SymantecRegistryDir="$SymantecSupportDir/Registry"
   SafetyMinderPlist="$SymantecSupportDir/Norton Safety Minder/Localization.bundle/Contents/Info.plist"
   VPNPlist="$SymantecSupportDir/$VPNAppName.app/Contents/Info.plist"
   WCIDEngineBundle="$SymantecSupportDir/WebFraud/WCIDEngine.bundle"
   WCIDEnginePlist="$WCIDEngineBundle/Contents/Info.plist"
   SupportPlists=`ls -d "$SymantecRegistryDir/"*".plist" 2>/dev/null | grep -ie "com\.symantec\.$LimitOutputPattern"`
   if [ -z "$LimitOutputPattern" ] ; then
      SupportPlists="$SupportPlists
`ls -d "$iAntiVirusPlist" "$SafetyMinderPlist" "$WCIDEnginePlist" 2>/dev/null`"
   fi
   Stage2TextToShow="Stage 2 builds and other components:
"
   SupportPlists=`echo "$SupportPlists" | sort -f | uniq | grep .`
   if [ "$SupportPlists" ] ; then
      echo "$Stage2TextToShow"
      Stage2TextToShow=""
      ProductTable=""
      for SupportPlist in $SupportPlists ; do
         # If iAntiVirus
         if [ "$SupportPlist" = "$iAntiVirusPlist" ] ; then
            VersionToAdd=`grep '>CFBundleShortVersionString<' -A1 "$iAntiVirusPlist" | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
            BuildToAdd=`grep -a -e '^ CFBundleVersion' "$iAntiVirusPlistStrings" | tail -n 1 | awk -F '"' '{print $2}'`
            [ -z "$BuildToAdd" ] && BuildToAdd=`iconv --from-code UTF-16 --to-code UTF-8 "$iAntiVirusPlistStrings" 2>/dev/null | grep -a -e 'CFBundleVersion' | tail -n 1 | awk -F '"' '{print $2}'`
            [ -z "$BuildToAdd" ] && BuildToAdd="build unknown"
            # Add product info to list of products
            ProductTable="$ProductTable
iAntiVirus $VersionToAdd ($BuildToAdd)"
         # Else if Norton Safety Minder 2.x
         elif [ "$SupportPlist" = "$SafetyMinderPlist" ] ; then
            # Add product name to list of products
            ProductTable="$ProductTable
Norton Safety Minder"
            # Add product version to list of products
            ProductTable="$ProductTable `grep '>FullVersion<' -A1 "$SupportPlist" | grep 'string>' | awk -F '>' '{print $2}' | awk -F '<' '{print $1}' | tail -n 1`"
         # Else if WCID engine
         elif [ "$SupportPlist" = "$WCIDEnginePlist" ] ; then
            WCIDParentDir=`dirname "$WCIDEngineBundle"`
            WCIDBundleBase=`basename "$WCIDEngineBundle"`
            # Add product info via ListInfoPlist to list of products
            ProductTable="$ProductTable
`ListInfoPlist -n "$WCIDParentDir" "$WCIDBundleBase" "WCID Engine"`"
            # Add product version to list of products
            ProductTable="$ProductTable `grep '>FullVersion<' -A1 "$SupportPlist" | grep 'string>' | awk -F '>' '{print $2}' | awk -F '<' '{print $1}' | tail -n 1`"
         else
            # Add product name to list of products
            ProductTable="$ProductTable
`grep '>en<' -A1 "$SupportPlist" | grep 'string>' | awk -F '>' '{print $2}' | awk -F '<' '{print $1}' | tail -n 1 | tr -d -c '[:alnum:] '`"
            # Add product version to list of products
            ProductTable="$ProductTable `grep '>Version<' -A1 "$SupportPlist" | grep 'string>' | grep -v '\*' | awk -F '>' '{print $2}' | awk -F '<' '{print $1}' | tail -n 1`"
         fi
      done
      # 3.0.15 - only show items that begin with aphanumeric characters (those which have Name keys in their plists)
      echo "$ProductTable" | grep '^[[:alnum:]]' | sort -f | uniq | awk '{printf "   %s\n\n", $0}'
   fi
   if [ -f "$VPNPlist" ] ; then
      if [ "$Stage2TextToShow" ] ; then
         echo "$Stage2TextToShow"
      fi
      VPNVersion=`cat "$VPNPlist" 2>/dev/null | tr '\015' '\012' | grep . | grep -A1 CFBundleShortVersionString | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      if [ -z "$VPNVersion" ] ; then
         VPNVersion="(could not determine version)"
      fi
      echo "   $VPNAppName $VPNVersion"
      echo
   fi
   # Show SACM version
   ShowSACMVersion
}

ShowKey()
{
   echo "$PlistContent" | grep -A1 -e '<key>'"$1"'</key>' | grep -ve '<key>'"$1"'</key>' | grep '<' | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'
}

ShowLooseInApplicationsFiles()
{
   # If a limiting pattern was specified, skip output
   [ "$LimitOutputPattern" ] && return
   LooseInApplicationsFolder="`ls -d "$PathToUseAsPrefix/Applications/iAntiVirus.app" "$PathToUseAsPrefix/Applications/Norton"* "$PathToUseAsPrefix/Applications/Symantec"* 2>/dev/null | grep -vE '/Applications/Norton Solutions$|/Applications/Symantec Solutions$' | sort -f`"
   if [ -n "$LooseInApplicationsFolder" ] ; then
      echo "Loose in /Applications$VolumeText:"
      echo
      for EachLoose in $LooseInApplicationsFolder ; do
         EachLooseBase=`basename "$EachLoose"`
         EachLooseDir=`dirname "$EachLoose"`
         EachLooseInfo=`ListInfoPlist "$EachLooseDir" "$EachLooseBase"`
         if [ "$EachLooseInfo" ] ; then
            echo "$EachLooseInfo"
         else
            echo "   $EachLooseBase`[ -L "$EachLoose" ] && echo " (symbolic link)"`"
         fi
         echo
      done
   fi
}

ShowRegistryInfoForLiveUpdate5()
{
   # Show LiveUpdate 5.x and earlier registry file info
   RegistryDirFull="$Path$RegistryDirLiveUpdate5"
   if [ -z "`ls "$RegistryDirFull"/* 2>/dev/null`" ] ; then
      if $DoShowMessageIfNotInstalled ; then
         echo "NOTE: No LiveUpdate 5 registry files were found$VolumeText."
         echo
      fi
      return
   fi
   RegistryFilesToShow=`ls -d "$RegistryDirFull"/* | sort -f | grep -ie "$RegistryDirFull/$LimitOutputPattern"`
   if [ -z "$RegistryFilesToShow" ] ; then
      if $DoShowMessageIfNotInstalled ; then
         echo "No LiveUpdate 5 registry files begin with pattern \"$LimitOutputPattern\""
         echo
      fi
      return
   fi
   echo "LiveUpdate 5 Registry files$LU5HeaderAddOn1${VolumeText}:"
   [ "$LU5HeaderAddOn2" ] && echo "$LU5HeaderAddOn2"
   echo
   for TheFile in $RegistryFilesToShow ; do
      FileName=`basename "$TheFile"`
      if [ -d "$TheFile" ] ; then
         InFolder=`ls "$TheFile" 2>/dev/null | sort -f`
         printf "%s (" "$FileName"
         if [ "$InFolder" ] ; then
            echo "folder):"
            printf "%s" "$InFolder" | awk '{print "   "$0}'
         else
            echo "empty folder)"
         fi
         echo
         continue
      fi
      echo "-- $FileName:"
      if [ "z$Option" = z-s ] ; then
         hexdump -v -C -s 0x16c -n 16 "$TheFile" | awk -F '|' '{printf "   "$2}'
         hexdump -v -C -s 0x34c -n 10 "$TheFile" | awk -F '|' '{printf "   "$2}'
         echo
      elif [ "z$Option" = z-X ] ; then
         echo
         hexdump -C -v "$TheFile" | grep '|'
      elif [ "z$Option" = z-x ] ; then
         echo
         hexdump -C -v "$TheFile" | grep '|' | awk '{printf substr(substr($0,1,length($0)-1),match($0,"\\|")+1)}'
         echo
      else
         ProductName=`hexdump -C -v -s 0x6c -n 64 "$TheFile" | grep '|' | awk '{printf substr(substr($0,1,length($0)-1),match($0,"\\\|")+1)}'`
         [ "`echo "z$ProductName" | grep '\.\.'`" ] && ProductName=`echo "z$ProductName" | awk '{print substr($0,2,match($0,"\\\.\\\.")-2)}'`
         ProductVersion=`hexdump -C -v -s 0x16c -n 64 "$TheFile" | grep '|' | awk '{printf substr(substr($0,1,length($0)-1),match($0,"\\\|")+1)}'`
         [ "`echo "z$ProductVersion" | grep '\.\.'`" ] && ProductVersion=`echo "z$ProductVersion" | awk '{print substr($0,2,match($0,"\\\.\\\.")-2)}'`
         Language=`hexdump -C -v -s 0x1ac -n 64 "$TheFile" | grep '|' | awk '{printf substr(substr($0,1,length($0)-1),match($0,"\\\|")+1)}'`
         [ "`echo "z$Language" | grep '\.\.'`" ] && Language=`echo "z$Language" | awk '{print substr($0,2,match($0,"\\\.\\\.")-2)}'`
         ItemSeqName=`hexdump -C -v -s 0x30c -n 64 "$TheFile" | grep '|' | awk '{printf substr(substr($0,1,length($0)-1),match($0,"\\\|")+1)}'`
         [ "`echo "z$ItemSeqName" | grep '\.\.'`" ] && ItemSeqName=`echo "z$ItemSeqName" | awk '{print substr($0,2,match($0,"\\\.\\\.")-2)}'`
         ItemSeqData=`hexdump -C -v -s 0x34c -n 64 "$TheFile" | grep '|' | awk '{printf substr(substr($0,1,length($0)-1),match($0,"\\\|")+1)}'`
         [ "`echo "z$ItemSeqData" | grep '\.\.'`" ] && ItemSeqData=`echo "z$ItemSeqData" | awk '{print substr($0,2,match($0,"\\\.\\\.")-2)}'`
         if $Verbose ; then
            echo "   ProductName:
      $ProductName
   Version:
      $ProductVersion
   Language:
      $Language
   ItemSeqName (Sequence Type):
      $ItemSeqName
   ItemSeqData (Sequence Number):
      $ItemSeqData"
         else
            echo "   $ProductName, $ProductVersion, $Language, $ItemSeqName, $ItemSeqData"
         fi
      fi
      echo
   done
}

ShowRegistryInfoForLiveUpdate6()
{
   local RegistryDirFull="$1"
   local RegistryFilesToShow="$2"
   # Show LiveUpdate 6 and later registry file info
   echo "LiveUpdate 6 `basename "$RegistryDirFull"` files$LU6HeaderAddOn1${VolumeText}:"
   [ "$LU6HeaderAddOn2" ] && echo "$LU6HeaderAddOn2"
   echo
   for TheFile in $RegistryFilesToShow ; do
      FileName=`basename "$TheFile"`
      if [ -d "$TheFile" ] ; then
         InFolder=`ls "$TheFile" 2>/dev/null | sort -f`
         printf "%s (" "$FileName"
         if [ "$InFolder" ] ; then
            echo "folder):"
            printf "%s" "$InFolder" | awk '{print "   "$0}'
         else
            echo "empty folder)"
         fi
         echo
         continue
      fi
      PlistContent=`cat "$TheFile" 2>/dev/null | tr '\015' '\012' | cat -v | grep '	.*<'`
      echo "-- $FileName:"
      if $DoShowFullPlist ; then
         echo
         echo "$PlistContent"
         echo
      else
         ProductName=`ShowKey ProductName`
         ProductVersionList=`ShowKey Version`
         # If this is an update that is not on LiveUpdate servers
         if [ -z "$ProductVersionList" ] ; then
            NonLiveUpdateHostName=`ShowKey NonLiveUpdateHostName`
            if [ "$NonLiveUpdateHostName" ] ; then
               echo "   ProductName (its updates reside on a non-LiveUpdate server):
      $ProductName
   NonLiveUpdateHostName:
      $NonLiveUpdateHostName
   LastModifiedCheck:
      `ShowKey LastModifiedCheck`
   NonLiveUpdateFileName:
      `ShowKey NonLiveUpdateFileName`
   TargetFileName:
      `ShowKey TargetFileName`"
            else
               echo "   NOTE: No version or host info is available"
            fi
            echo
            continue
         fi
         LanguageList=`ShowKey Language`
         ItemSeqNameList=`ShowKey ItemSeqName`
         ItemSeqDataList=`ShowKey ItemSeqData`
         Row=1
         for ProductVersion in $ProductVersionList ; do
            Language=`TableLookup "$LanguageList" $Row`
            ItemSeqName=`TableLookup "$ItemSeqNameList" $Row`
            ItemSeqData=`TableLookup "$ItemSeqDataList" $Row`
            let Row=$Row+1
            if [ "z$Option" = z-s ] ; then
               echo "   $ProductVersion -> $ItemSeqData"
            elif $Verbose ; then
               echo "   Product Name:
      $ProductName
   Version:
      $ProductVersion
   Language:
      $Language
   ItemSeqName (Sequence Type):
      $ItemSeqName
   ItemSeqData (Sequence Number):
      $ItemSeqData"
            else
               echo "   $ProductName, $ProductVersion, $Language, $ItemSeqName, $ItemSeqData"
            fi
         done
      fi
      echo
   done
}

ShowRegistryInfoForLiveUpdate6All()
{
   # Show LiveUpdate 6 and later registry file info
   ActiveRegistryDirFull="$Path$ActiveRegistryDirLiveUpdate6"
   NewRegistryDirFull="$Path$NewRegistryDirLiveUpdate6"
   if [ -z "`ls "$ActiveRegistryDirFull"/*.plist "$NewRegistryDirFull"/*.plist 2>/dev/null`" ] ; then
      if $DoShowMessageIfNotInstalled ; then
         echo "NOTE: No LiveUpdate 6 registry plists were found$VolumeText."
         echo
      fi
      return
   fi
   ActiveRegistryFilesToShow=`ls -d "$ActiveRegistryDirFull"/* 2>/dev/null | sort -f | grep -ie "$ActiveRegistryDirFull/$LimitOutputPattern"`
   NewRegistryFilesToShow=`ls -d "$NewRegistryDirFull"/* 2>/dev/null | sort -f | grep -ie "$NewRegistryDirFull/$LimitOutputPattern"`
   if [ -z "$ActiveRegistryFilesToShow$NewRegistryFilesToShow" ] ; then
      if $DoShowMessageIfNotInstalled ; then
         echo "No LiveUpdate 6 registry plists begin with pattern \"$LimitOutputPattern\""
         echo
      fi
      return
   fi
   [ "$ActiveRegistryFilesToShow" ] && ShowRegistryInfoForLiveUpdate6 "$ActiveRegistryDirFull" "$ActiveRegistryFilesToShow"
   [ "$NewRegistryFilesToShow" ] && ShowRegistryInfoForLiveUpdate6 "$NewRegistryDirFull" "$NewRegistryFilesToShow"
}

ShowSACMVersion()
{
   # Usage:     ShowSACMVersion
   # Summary:   Displays version of AddGroups if installed and the
   #            number of installed SMac man pages.
   #
   [ -z "`echo "Symantec Administration Console" | grep -ie "^$LimitOutputPattern"``echo "SACM" | grep -ie "^$LimitOutputPattern"`" ] && return
   RockefellerCenterNotInstalled=true
   # If early build of Symantec Administration Console server is installed
   if [ -f "$PathToUseAsPrefix/Library/Application Support/Symantec/SMACS/CmdExec/Admin/AddGroups" ] ; then
      printf "   Symantec Administration Console "
      "$PathToUseAsPrefix/Library/Application Support/Symantec/SMACS/CmdExec/Admin/AddGroups" -V | head -n 1 | awk '{print substr($0,length($0)-30)}' | awk '{print $3 $4" - Server"}'
      echo
      RockefellerCenterNotInstalled=false
   fi
   # If early build of Symantec Administration Console client is installed
   if [ -f "$PathToUseAsPrefix/Library/Application Support/Symantec/SMACS/Commands/Admin/AddGroups" ] ; then
      printf "   Symantec Administration Console "
      "$PathToUseAsPrefix/Library/Application Support/Symantec/SMACS/Commands/Admin/AddGroups" -V | head -n 1 | awk '{print substr($0,length($0)-30)}' | awk '{print $3 $4" - Client"}'
      echo
      RockefellerCenterNotInstalled=false
   fi
   # If later build of Symantec Administration Console server is installed
   if [ -f "$PathToUseAsPrefix/Library/Application Support/Symantec/SMac/CmdExec/Admin/AddGroups" ] ; then
      printf "   Symantec Administration Console "
      AGVerStr="`"$PathToUseAsPrefix/Library/Application Support/Symantec/SMac/CmdExec/Admin/AddGroups" -V | head -n 1 | awk '{print substr($0,length($0)-30)}'`"
      if [ `echo "$AGVerStr" | grep -c "manually built"` != 0 ] ; then
         echo "$AGVerStr - Server"
      else
         echo "$AGVerStr" | awk '{print $3 $4" - Server"}'
      fi
      echo
      RockefellerCenterNotInstalled=false
   fi
   # If later build of Symantec Administration Console client is installed
   if [ -f "$PathToUseAsPrefix/Library/Application Support/Symantec/SMac/Commands/Admin/AddGroups" ] ; then
      printf "   Symantec Administration Console "
      AGVerStr="`"$PathToUseAsPrefix/Library/Application Support/Symantec/SMac/Commands/Admin/AddGroups" -V | head -n 1 | awk '{print substr($0,length($0)-30)}'`"
      if [ `echo "$AGVerStr" | grep -c "manually built"` != 0 ] ; then
         echo "$AGVerStr - Client"
      else
         echo "$AGVerStr" | awk '{print $3 $4" - Client"}'
      fi
      echo
      RockefellerCenterNotInstalled=false
   fi
   ManPagesFound=`find "$PathToUseAsPrefix/usr/local/man" "$PathToUseAsPrefix/usr/share/man" 2>/dev/null | grep -c symc`
   # If Symantec Administration Console man page(s) found
   if [ $ManPagesFound != 0 ] ; then
      if [ $ManPagesFound = 1 ] ; then
         echo "   Symantec Administration Console - $ManPagesFound man page file is installed."
      else
         echo "   Symantec Administration Console - $ManPagesFound man page files are installed."
      fi
      echo
      RockefellerCenterNotInstalled=false
   fi
   if $RockefellerCenterNotInstalled ; then
      if $DoShowMessageIfNotInstalled ; then
         echo "Symantec Administration Console is not installed$VolumeText."
         echo
      fi
   elif [ $ManPagesFound = 0 ] ; then
      echo "   Symantec Administration Console - No man page files are installed."
      echo
   fi
}

ShowUsage()
{
   # Usage:     ShowUsage [$1 [$2]]
   # Arguments: $1 = Exit code.
   #            $2 = Error message to display before showing usage.
   # Summary:   Displays script usage. If an exit code is passed,
   #            script is exited with that value.
   #
   if [ "$2" ] ; then
      echo
      echo "$2"
      echo
   fi
   ShowHelp | grep "^Usage.*:"
   [ "$2" ] && echo
   [ -n "$1" ] && exit "$1"
}

TableLookup()
{
   # Usage:     TableLookup [-k] Table [Row [Column [Delimiter]]]
   # Arguments: Table       String table with rows delimited by newlines and
   #                        columns delimited by tabs. Blank rows are omitted
   #                        and carriage returns are converted to newlines.
   #            Row         Row number in range 1 thru number of rows in table.
   #            Column      Column number.
   #            Delimiter   Column delimiter.
   #            -k          If -k is passed as the first argument, blank lines
   #                        are not ignored and carriage returns are not converted.
   # Summary:   If Column is not passed, non-blank row Row of table is printed.
   #            If Row > number of rows in table or less than 1, nothing is printed.
   #            If Row is "", column Column of all rows is printed.
   #            If Row and Column are "", non-blank lines Table are printed.
   #            If Delimiter is not "", then it is used as the column delimiter;
   #            otherwise, tab is used as the delimiter.
   #
   local TLColumn
   local TLDelimiter
   local TLRemoveBlankLines=true
   local TLRow
   local TLStringToShow=""
   local TLTable
   if [ "z$1" = z-k ] ; then
      shift
      TLRemoveBlankLines=false
   fi
   TLTable="$1"
   TLRow="$2"
   TLColumn="$3"
   TLDelimiter="$4"
   $TLRemoveBlankLines && TLTable=`printf "%s" "$TLTable" | tr '\015' '\012' | grep .`
   [ -z "$TLDelimiter" ] && TLDelimiter="	"
   if [ -z "$TLTable" ] ; then
      return
   elif [ -z "$TLRow$TLColumn" ] ; then
      TLStringToShow=`printf "%s" "$TLTable"`
   elif [ -z "$TLRow" ] ; then
      [ "`echo "0$TLColumn" | tr -d '[:digit:]'`" ] && return
      TLStringToShow=`printf "%s" "$TLTable" | awk -F "$TLDelimiter" -v NUM=$TLColumn '{print $NUM}'`
   elif [ "`echo "0$TLRow" | tr -d '[:digit:]'`" ] ; then
      return
   elif [ $TLRow -lt 1 -o $TLRow -gt `printf "%s" "$TLTable" | grep -c .` ] ; then
      return
   elif [ -z "$TLColumn" ] ; then
      TLStringToShow=`printf "%s" "$TLTable" | head -n "$TLRow" | tail -n 1`
   elif [ "`echo "0$TLColumn" | tr -d '[:digit:]'`" ] ; then
      return
   else
      TLStringToShow=`printf "%s" "$TLTable" | head -n "$TLRow" | tail -n 1 | awk -F "$TLDelimiter" -v NUM=$TLColumn '{print $NUM}'`
   fi
   $TLRemoveBlankLines && TLStringToShow=`printf "%s" "$TLStringToShow" | grep .`
   [ "$TLStringToShow" ] && printf "%s\n" "$TLStringToShow"
}

# *** Beginning of Commands to Execute ***

[ $# = 0 ] && clear
ScriptPath=`ShowFullFilePath "$0" -P`
[ ! -e "$ScriptPath" ] && ExitScript 99 "WARNING: ScriptPath definition failed. Script aborted."
FullScriptName=`basename "$ScriptPath"`
ScriptDir=`dirname "$ScriptPath"`
LUXToolsDir="$ScriptDir/../../LiveUpdateTools/LUX/NFM"
ViewDatastoreProgram="$LUXToolsDir/ViewDatastore.command"
ProcessArguments "$@"
echo
ListOfPaths=`echo "$ListOfPaths" | sort -f | uniq`
ListOfPathsToUse=""
IFS='
'
for Path in $ListOfPaths ; do
   ListOfPathsToUse="$ListOfPathsToUse
`ShowFullFilePath -P "$Path"`"
done
ListOfPathsToUse=`echo "$ListOfPathsToUse" | sort -f | uniq | grep /`
PathNumber=1
for Path in $ListOfPathsToUse ; do
   if [ $PathNumber -gt 1 ] ; then
      echo "--------------------------------------------------------------------------------"
      echo
   fi
   let PathNumber=$PathNumber+1
   if [ "$Path" = / ] ; then
      VolumeText=" on boot volume"
   elif [ "`dirname "$Path"`" = /Volumes ] ; then
      VolumeText=" on volume \"`basename "$Path"`\""
   else
      VolumeText=" in folder \"`basename "$Path"`\""
   fi
   $RegistryInfoBefore || ShowInstalledApplicationInfo
   $DoShowRegistryInfo || continue
   ShowRegistryInfoForLiveUpdate5
   ShowRegistryInfoForLiveUpdate6All
   [ -f "$ViewDatastoreProgram" ] && "$ViewDatastoreProgram" -D "$Path" -B $ViewDatastoreOption
   $DoShowAppInfo || continue
   $RegistryInfoBefore && ShowInstalledApplicationInfo
done
exit 0

# *** End of Commands to Execute ***
