#!/bin/sh
  Version=2.0.59
# Author:       Corey Swertfager, NortonLifeLock Inc.
# Created:      12/08/2008
# Modified:     12/21/2020
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header start
# //////////////////////
#
# PROPRIETARY / CONFIDENTIAL.
# Use of this product is subject to license terms.
# Copyright (c) 2020 NortonLifeLock Inc.
# All rights reserved.
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header stop
# //////////////////////
#
# Usage:   See ShowHelp() function.
#
# Summary: See ShowHelp() function.
#
# History: 1.00 - Created.
#          1.01 - Added -f and -i options.
#                 Made -e option the default.
#                 Renamed master list to Paths-MasterList.txt.
#          1.02 - Updated Paths-MasterList.txt.
#                 Added -u option.
#                 Now checks to make sure framework binaries are not signed
#                 by codesign.
#          1.03 - Removed extra blank line from framework codesign output.
#                 Updated Paths-MasterList.txt.
#                 Now parses list entries before #.
#                 Added --nocodesign as an option in file list's
#                 SignaturePath field (third field).
#          1.04 - Updated DefineCurrentUserAndDesktop for OS 10.1 variant.
#          1.05 - Updated Paths-MasterList.txt.
#                 Set SignaturePath to "" when --nocodesign passed.
#                 Now uses plist.sig if corresponding plist sig file is not
#                 found (example: if x.plist is passed but x.sig does not
#                 exist in the same folder, x.plist.sig will be used).
#          1.06 - Updated Paths-MasterList.txt.
#          1.07 - Updated Paths-MasterList.txt.
#                 If Argument_For_VerifyTCSig or SignaturePath field in
#                 file list begins with a #, it is treated as a comment.
#          1.08 - Added -I option: show errors for files that are not installed.
#                 Made -i the default option.
#          1.09 - Updated Paths-MasterList.txt based on IPS Engine 3.0 build 6.
#          1.10 - ScriptPath definition added for better script linking.
#          1.11 - Reconfigured tr command error output.
#          1.12 - Updated Paths-MasterList.txt: added Norton DNS.
#          1.13 - Replaced DefineCurrentUserAndDesktop with DefineCurrentUserDesktopAndOS.
#          1.14 - Added volume/folder option.
#                 Removed CurrentUserTempFile/ProcessArgumentsCalled.
#          1.15 - Modified for case-sensitive volume compatibility.
#                 Replaced ShowFullFilePath function with version 1.0.2.
#          1.16 - Updated Paths-MasterList.txt based on NIS 5 build 19 boms.
#                 Replaced VerifyTCSig with Perforce revision 15.
#                 Added display of error totals at end.
#          1.17 - Updated how VerifyTCSig is called for NIS 5.
#                 Updated Paths-MasterList.txt:
#                    Changed /Versions/A to /Versions/Current.
#                    Changed /Versions/B to /Versions/Current.
#                 Added VerifyTCSig Perforce revision 17.
#                 Deleted VerifyTCSig Perforce revision 13.
#          1.18 - Updated Paths-MasterList.txt for NCO 1.3.4/1.4.4:
#                    Added libgecko4parsers.dylib.
#                    Added libnortonconfidential4.dylib.
#                    Added libnortonsafetyminder4.dylib.
#          1.19 - Added VerifyTCSig Perforce revision 14 to address bus
#                 error when running on OS 10.4.
#          1.20 - Updated Paths-MasterList.txt for NIS 5:
#                    Added FirewallUI.framework.
#          1.21 - Updated Paths-MasterList.txt for NIS 5:
#                    Added WebFraud.bundle.
#          1.22 - Added to Paths-MasterList.txt for NIS 5 and beyond:
#                    libgecko5parsers.dylib
#                    libgecko6parsers.dylib
#                    libnortonconfidential5.dylib
#                    libnortonconfidential6.dylib
#                    libnortonsafetyminder5.dylib
#                    libnortonsafetyminder6.dylib
#                    SymLicensing_embedded.framework
#                    UIALogLUPlugIn.bundle
#          1.23 - Added to Paths-MasterList.txt:
#                    StuffIt.framework
#                    StuffItCore.framework
#                 Removed FileListsInProgress folder.
#          1.24 - Disabled in Paths-MasterList.txt:
#                    StuffIt.framework (won't be signed)
#                    StuffItCore.framework (won't be signed)
#          1.25 - Added to Paths-MasterList.txt for NIS 5 and beyond:
#                    libgecko7parsers.dylib
#                    libnortonconfidential7.dylib
#                    libnortonconfidentialwp5.dylib
#                    libnortonconfidentialwp6.dylib
#                    libnortonconfidentialwp7.dylib
#          1.26 - 12/07/2011:
#                 Added to Paths-MasterList.txt:
#                    libgecko8parsers.dylib
#                    libgecko9parsers.dylib
#                    libnortonconfidential8.dylib
#                    libnortonconfidential9.dylib
#                    libnortonconfidentialwp8.dylib
#                    libnortonconfidentialwp9.dylib
#                    libwpsapi.dylib
#                    NATDaemonPlugin Alerting.bundle
#                    NATDaemonPlugin Logging.bundle
#                    NATDaemonPlugin.bundle
#                    NATRemoteLock.app
#                    NortonM Alerting.bundle
#                    NortonM Logging.bundle
#                    NortonM.bundle
#                    SymOxygen.framework
#          1.27 - 12/14/2011:
#                 Disabled in Paths-MasterList.txt:
#                    libwpsapi.dylib
#          1.28 - 02/05/2012:
#                 DefaultListName can now be in ScriptDir.
#          1.29 - 07/02/2012 - Corey Swertfager:
#                 * If Argument_For_VerifyTCSig is omitted for an entry
#                   and Path has the extension .dylib, --binary will be
#                   passed to VerifyTCSig.
#                 * Changed dylib paths in Paths-MasterList.txt to use
#                   wildcard for:
#                      libgeckoparsers*.dylib
#                      libnortonconfidential*.dylib
#                      libnortonsafetyminder*.dylib
#                      libwebkit*parsers.dylib
#                 * Added check for ActiveRegistry folder when determining
#                   if NIS 5 or later (LiveUpdate 6 or later) is installed.
#                 * Added capability to check paths that end with /Versions/Current.
#                 * Appended /Versions/Current to paths for OS 10.6 codesign compatibility
#                   in Paths-MasterList.txt for:
#                      FirewallUI.framework
#                      SymIR.framework
#                      SymLicensing.framework
#                      SymLicensing_embedded.framework
#                      SymOxygen.framework
#          1.30 - 07/17/2012 - Corey Swertfager:
#                 Added to Paths-MasterList.txt:
#                    /Applications/Norton Zone.app
#                    /Applications/Norton Zone.app/Contents/Frameworks/mach_inject_bundle.framework/Versions/Current
#                    /Applications/Norton Zone.app/Contents/Frameworks/mach_inject_bundle.framework/Versions/Current/Resources/mach_inject_bundle_stub.bundle
#                    /Applications/Norton Zone.app/Contents/Frameworks/SymBase.framework/Versions/Current
#                    /Applications/Norton Zone.app/Contents/Frameworks/SymOxygen.framework/Versions/Current
#          1.31 - 08/03/2012 - Corey Swertfager:
#                 Added/updated the following paths in Paths-MasterList.txt:
#                    /Library/Frameworks/mach_inject_bundle.framework
#                    /Library/Frameworks/mach_inject_bundle.framework/Versions/Current/Resources/mach_inject_bundle_stub.bundle
#                    /Library/Application Support/Symantec/Mexico/FinderPlugin.bundle
#          1.32 - 10/16/2012 - Corey Swertfager:
#                 Added/updated the following paths in Paths-MasterList.txt:
#                    /Applications/Symantec Solutions/Norton Internet Security.app
#                    /Library/Application Support/Symantec/SymQuickMenu/NisQuickMenu.plugIn
#                    /Library/Application Support/Symantec/SymUIAgent/PlugIns/BrowserFrameworkUIAgent.bundle
#                    /Library/Internet Plug-Ins/NortonInternetSecurityBF.plugin
#                    /Library/Internet Plug-Ins/NortonInternetSecurityBF.plugin/Contents/PlugIns/NortonSafeSurfing.bundle
#          1.33 - 03/12/2013 - Corey Swertfager:
#                 * Set exit codes as follows:
#                       0 = no installation errors found
#                       1 = admin password authentication failed and aborted script
#                      20 = signature errors were found
#                      78 = fatal error caused script to exit prematurely
#                 * Added watermark ID.
#          1.34 - 05/24/2013 - Corey Swertfager:
#                 Etrack incident 3206961: Bypasses codesign validation of the following
#                 if Mac OS version is 10.8 or greater:
#                    /Library/Application Support/Symantec/AntiVirus/SymAPComm.kext
#          2.0.0 - 06/30/2013 - Corey Swertfager:
#                  * ProcessArguments 1.0.1 is now used to process arguments to allow
#                    options to be combined in one argument that begins with a single
#                    hyphen (e.g., passing -ab to script would be equivalent to passing
#                    -a and -b separately; --ab would be treated as a single option).
#                  * Updated Paths-MasterList.txt based on NIS 6 build 75 boms.
#          2.0.1 - 07/10/2013 - Corey Swertfager:
#                  * Updated Paths-MasterList.txt based on NIS 6 build 92 boms; added:
#                       /Library/Application Support/Symantec/Silo/NFM/Daemon/SymDaemon.bundle/Contents/PlugIns/NortonM.bundle
#                       /Library/Application Support/Symantec/Silo/NFM/SymUIAgent/PlugIns/NortonM Alerting.bundle
#                       /Library/Application Support/Symantec/Silo/NFM/SymUIAgent/PlugIns/NortonM Logging.bundle
#                  * Removed:
#                       /Library/Application Support/Symantec/IntrusionPrevention/CurrentEngine/SymIPS.kext
#                       /Library/Application Support/Symantec/IntrusionPrevention/NewEngine/SymIPS.kext
#                       /System/Library/Extensions/SymInternetSecurity.kext
#                       /System/Library/Extensions/SymIPS.kext
#          2.0.2 - 07/14/2013 - Corey Swertfager:
#                  * Updated Paths-MasterList.txt based on NIS 6 build 106 boms; added:
#                       /Library/Application Support/Symantec/Silo/NFM/Frameworks/libnlucallback.dylib
#                       /Library/Application Support/Symantec/Silo/NFM/SymUIAgent/PlugIns/NortonM Logging.bundle
#                  * Removed:
#                       /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libnlucallback.dylib
#          2.0.3 - 07/17/2013 - Corey Swertfager:
#                  * Updated Paths-MasterList.txt based on NIS 6 build 117 boms; added:
#                       /Library/Application Support/Symantec/Silo/NFM/Daemon/SymDaemon.bundle/Contents/PlugIns/AutoBlock.bundle
#          2.0.4 - 09/05/2013 - Corey Swertfager:
#                  * Updated Paths-MasterList.txt based on NIS 5.6 stage 2 build 10 boms:
#                    added:
#                       /Library/Extensions/FileSecurity.kext
#                       /Library/Extensions/SymAPComm.kext
#                       /Library/Extensions/SymFirewall.kext
#                       /Library/Extensions/SymInternetSecurity.kext
#                       /Library/Extensions/SymIPS.kext
#                       /Library/Extensions/SymPersonalFirewall.kext
#                    removed:
#                       /Library/Application Support/Symantec/AntiVirus/SymAPComm.kext
#                       /System/Library/Extensions/SymPersonalFirewall.kext
#                  * Added file list SEP-Tesla.txt.
#                  * Added --SEPTelsa option to use SEP-Tesla.txt file list.
#          2.0.5 - 09/05/2013 - Corey Swertfager:
#                  * Changed --SEPTelsa option to --SEPTesla.
#          2.0.6 - 09/11/2013 - Corey Swertfager:
#                  * Added KextsForMavericks file list variable for verifying kexts
#                    using kextutil instead of codesign when running OS 10.9 or later;
#                    those kexts are ignored if running OS 10.8.x or earlier.
#                  * Updated Paths-MasterList.txt:
#                    added:
#                       /usr/local/lib/libecomlodr.dylib (only checked if NIS 6 or later is installed)
#                    removed:
#                       /Library/Application Support/Symantec/Silo/NFM/AntiVirus/Definitions/Engine/libecomlodr.dylib
#          2.0.7 - 09/13/2013 - Corey Swertfager:
#                  * Updated how verification program name is displayed.
#          2.0.8 - 09/25/2013 - Corey Swertfager:
#                  * Updated Paths-MasterList.txt based on NIS 6 build 190 boms:
#                    added:
#                       /Library/Application Support/Symantec/Silo/NFM/SymUIAgent/Norton.app
#                    removed:
#                       /Library/Application Support/Symantec/Silo/NFM/Branding/NFMProductUI.bundle
#          2.0.9 - 12/03/2013 - Corey Swertfager:
#                  * Changed logic for determining which version of VerifyTCSig to use.
#                  * Added VerifyTCSig 3.1r1, Perforce revision 4.
#                  * Updated Paths-MasterList.txt based on NIS 6 build 270 boms:
#                    added:
#                       /Library/Application Support/Symantec/Silo/NFM/SymUIAgent/PlugIns/LiveUpdateLog.bundle
#                       /Library/Extensions/NortonForMac.kext
#          2.0.10 - 12/05/2013 - Corey Swertfager:
#                   * Adjusted the way paths for Mavericks kexts are filtered.
#                   * Removed KextsForMavericks variable.
#          2.0.11 - 01/15/2014 - Corey Swertfager:
#                   * Added the following to SEP-Tesla.txt:
#                        /Applications/Symantec Solutions/SmallScanner.app
#          2.0.12 - 02/10/2014 - Corey Swertfager:
#                   * Removed VerifySignedFilesSupport/VerifyTCSig/1* to avoid symbols appearing.
#                   * VerifyTCSig is now only used when NIS 6 is installed.
#                   * Updated Paths-MasterList.txt: revised comment for:
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libnlucallback.dylib
#          2.0.13 - 02/21/2014 - Corey Swertfager:
#                   * Replaced VerifyTCSig with Perforce revision 5.
#          2.0.14 - 06/10/2014 - Corey Swertfager:
#                   * Added secondary path condition to target verification of primary path
#                     only when secondary path exists. See Paths-MasterList.txt for usage.
#                   * Updated Paths-MasterList.txt based on NIS 6.1 build 61 boms:
#                     Now validates signatures for these only if "/Applications/Norton Security.app" is installed:
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libAntiPhishingCallBack.dylib
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libcx_lib.so
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libduluxcallback.so.2.0.0
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/liblux.so.2
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libLuxCustomerLogger.so
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libnlucallback.dylib
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libpatchapp.so
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libSymDltCl.so
#                   * Changed faulty variable assignment from ' ="' to '="'.
#          2.0.15 - 06/13/2014 - Corey Swertfager:
#                   * Added file list SEP-Ferrari.txt (Etrack 3536763).
#                   * Added --SEPFerrari option to use SEP-Ferrari.txt file list.
#          2.0.16 - 06/19/2014 - Corey Swertfager:
#                   * Removed the following from SEP-Ferrari.txt:
#                        /usr/local/lib/libecomlodr.dylib
#          2.0.17 - 07/02/2014 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on NIS 6.1 build 78 boms:
#                        /Library/Services/Norton for Mac.service
#          2.0.18 - 07/30/2014 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on NIS 5.7 build 8 boms:
#                        /Library/Application Support/Symantec/NortonUpdate/NortonUpdate.app
#                        /Library/Application Support/Symantec/SymUIAgent/PlugIns/NortonUpdateAgentAlerting.bundle
#          2.0.19 - 08/10/2014 - Corey Swertfager:
#                   * Replaced DefineCurrentUserDesktopAndOS function with version 1.0.1.
#          2.0.20 - 08/19/2014 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on NIS 5.7 build 19 boms:
#                        /Library/Application Support/Symantec/Daemon/SymDaemon.bundle/Contents/PlugIns/NortonUpdateAgentPlugin.bundle
#                        /Library/Application Support/Symantec/FSD/bin/symfsdx
#                        /Library/Application Support/Symantec/IPUA/NortonProductUpdateApp.app
#          2.0.21 - 08/25/2014 - Corey Swertfager:
#                   * Corrected revision date to 2/20/2014 in comment for:
#                        //depot/Mac_Shared_Trunk/trunk/BuildTools/bin/VerifyTCSig
#          2.0.22 - 09/09/2014 - Corey Swertfager:
#                   * To address Etrack 3613868, VerifyTCSig is run only when the
#                     following exists:
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate
#          2.0.23 - 12/29/2014 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on SCS build 20 boms:
#                        /Applications/Symantec Cloud Security.app
#                        /Library/Application Support/Symantec/Silo/NFM/CoreUI/CoreUI.bundle
#                        /Library/Application Support/Symantec/Silo/NFM/Frameworks/SymUninstaller.framework/Versions/Current
#                        /Library/Application Support/Symantec/Silo/NFM/SymUIAgent/Symantec.app
#                        /Library/Application Support/Symantec/Silo/NFM/Uninstaller/NFMUninstallUI.bundle
#                        /Library/Services/Symantec Cloud Security.service
#          2.0.24 - 01/05/2015 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on SCS 7.0 build 22 boms:
#                        /Library/Application Support/Symantec/Silo/NFM/Uninstaller/NortonMRemoteUninstall.bundle
#          2.0.25 - 02/09/2015 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on SCS 7.0 build 42 boms:
#                        /Library/Application Support/Symantec/Silo/NFM/ClientSDK/FeatureService.dylib
#                        /Library/Application Support/Symantec/Silo/NFM/ClientSDK/libcx_lib.so
#                        /Library/Application Support/Symantec/Silo/NFM/ClientSDK/NFMFeatureService.dylib
#                        /Library/Application Support/Symantec/Silo/NFM/Daemon/SymDaemon.bundle/Contents/PlugIns/csdk.bundle
#                        /Library/Application Support/Symantec/Silo/NFM/Daemon/SymDaemon.bundle/Contents/PlugIns/EPMP.bundle
#                        /Library/Application Support/Symantec/Silo/NFM/Daemon/SymDaemon.bundle/Contents/PlugIns/LiveUpdateJobPlugin.bundle
#                        /Library/Application Support/Symantec/Silo/NFM/Frameworks/csdk.dylib
#                        /Library/Application Support/Symantec/Silo/NFM/Frameworks/libcx_ipc.so
#                        /Library/Application Support/Symantec/Silo/NFM/Frameworks/libcx_lib.so
#          2.0.26 - 04/08/2015 - Corey Swertfager:
#                   * Added -r option to report every "resource envelope is obsolete" error.
#                   * Individual "resource envelope is obsolete" errors are no longer shown
#                     unless -r is passed.
#          2.0.27 - 07/08/2015 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on NFM 7.0 build 242 boms:
#                        /Applications/Symantec Unified Endpoint Protection.app
#          2.0.28 - 08/05/2015 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on NFM 7.0 build 264 boms:
#                        /Library/Services/Symantec UEP.service
#          2.0.29 - 08/31/2015 - Corey Swertfager:
#                   * Added to Paths-MasterList.txt based on NFM 7.0 build 311 boms:
#                        /Library/Application Support/Symantec/Silo/NFM/LiveUpdate/lib/libduluxcallback.so.2.3.0
#          2.0.30 - 11/19/2015 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on SEP 7.0 Agile-branch 2502 boms.
#          2.0.31 - 11/30/2015 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on SEP 7.0 Agile-branch 80 boms.
#          2.0.32 - 03/30/2016 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on NFM 7.1 build 22 boms.
#          2.0.33 - 08/11/2016 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on NFM 7.2 build 28_23 boms.
#          2.0.34 - 02/07/2017 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on NFM 7.3.1 build 18 and
#                     NFM 7.4 build 25 boms on on Norton WiFi Privacy (NWP) 1.0 build 71.
#          2.0.35 - 04/12/2017 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on NFM 7.4 build 140 boms.
#          2.0.36 - 06/07/2017 - Corey Swertfager:
#                   * Added support for use of wildcards in Paths-MasterList.txt.
#                   * Updated some paths in Paths-MasterList.txt to use wildcards.
#          2.0.37 - 10/16/2017 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on NFM 7.6 build 53 BOMs.
#          2.0.38 - 02/12/2018 - Davee Nguyen:
#                   * Updated Paths-MasterList.txt based on SEPC 8.0 build 408 BOMs.
#          2.0.39 - 07/20/2018 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt to include libsymavping.dylib
#                     (Jira MACSYS-4030).
#          2.0.40 - 07/23/2018 - Corey Swertfager:
#                   * Added VerifyTCSig 4.0d1 (Jira MACSYS-4030).
#                   * Updated help.
#          2.0.41 - 08/16/2018 - Corey Swertfager:
#                   * Updated VerifyTCSig 4.0d1 with revision 4.
#          2.0.42 - 08/24/2018 - Corey Swertfager:
#                   * Added VerifyTCSigBypassPatternKey to bypass VerifyTCSig
#                     verification of paths that match the associated pattern.
#                   * Updated Paths-MasterList.txt to exclude paths that contain
#                     Frameworks/libswift from VerifyTCSig verification (Jira
#                     MACSYS-4452 and MACSYS-4453).
#          2.0.43 - 09/29/2018 - Corey Swertfager:
#                   * Added check for Symantec as authority in Apple Swift
#                     libraries (MACSYS-4866).
#          2.0.44 - 01/14/2019 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt based on NFM 8.2 build 116 BOMs.
#          2.0.45 - 06/24/2019 - Corey Swertfager:
#                   * Added support for multiple Silo directories.
#                   * Updated Paths-MasterList.txt: changed /NFM/ to /*/.
#          2.0.46 - 06/24/2019 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt: added:
#                        /Library/Services/NortonDocktile.bundle
#          2.0.47 - 06/26/2019 - Corey Swertfager:
#                   * No longer uses kextutil to check kexts.
#                   * Added notarization check for kexts in /Library/Extensions
#                     when running OS 10.14 or later.
#          2.0.48 - 06/26/2019 - Corey Swertfager:
#                   * Now checks kext notarization only if NFM Silo exists.
#          2.0.49 - 06/26/2019 - Corey Swertfager:
#                   * Now checks kext notarization only if Silo/NFM/LiveUpdate exists.
#          2.0.50 - 07/18/2019 - Corey Swertfager:
#                   * Updated Paths-MasterList.txt: removed redundant frameworks
#                     and disabled paths.
#          2.0.51 - 09/12/2019 - Corey Swertfager:
#                   * Added detailed output from codesign when it returns non-zero
#                     or when -f is passed to this script.
#                   * Added md5 output for CodeResourcesFile in kexts when codesign
#                     returns non-zero.
#          2.0.52 - 04/03/2020 - Corey Swertfager:
#                  * Changed Author company to NortonLifeLock Inc.
#          2.0.53 - 07/28/2020 - Corey Swertfager:
#                   * Added macOS 11+ support.
#          2.0.54 - 11/04/2020 - Corey Swertfager:
#                   * Added copyright section.
#          2.0.55 - 12/21/2020 - Corey Swertfager:
#                   * Updated copyright section.
#          2.0.56 - 09/23/2021 - Venkatalakshmi:
#                   * Updated Paths-MasterList.txt: Added NSM bundle and app for RebootlessLite feature in 8.7.1a
#          2.0.57 - 03/29/2022 - Earth:
#                   * Updated VerifySignTools for ARM support
#          2.0.58 - 05/09/2022 - Earth:
#                   * Created Path-MasterList-Universal.txt and added the condition to use new file for Universal build
#          2.0.59 - 04/02/2022 - Venkatalakshmi.S:
#                   * Updated Nlok VerifySign tools for ARM support build/ Project North Star and future builds
# *** Variable Initializations ***

PATH=/bin:/sbin:/usr/bin:/usr/sbin
AppleVerifyProgramName=codesign
DefaultListName=Paths-MasterList.txt
DefaultArmListName=Paths-MasterList-Universal.txt
ExitCodeWhenSignatureErrorsFound=20
FileListsDirName=FileLists
LANG=""
SymantecSupportDir="/Library/Application Support/Symantec"
ActiveLiveUpdateRegistryFolder="$SymantecSupportDir/LiveUpdate/ActiveRegistry"
NewLiveUpdateRegistryFolder="$SymantecSupportDir/LiveUpdate/NewRegistry"
SiloRootDir="$SymantecSupportDir/Silo"
PublicVersion=false
ResourceEnvelopeObsoleteError="resource envelope is obsolete"
SEPFerrariList="SEP-Ferrari.txt"
SEPTeslaList="SEP-Tesla.txt"
ShowQuitMessage=false
SupportDirName=VerifySignedFilesSupport
SymantecVerifyProgramName=VerifyTCSig
# From Perforce: //depot/Mac_Shared_Trunk/trunk/BuildTools/bin/VerifyTCSig
#                Revision 5, 2/20/2014 - 3.1r1
SymantecVerifyProgramPathOld="VerifyTCSig/MST-3.1r1-005/$SymantecVerifyProgramName"
# From Perforce: //depot/Mac_Products/Core_Security/Norton_for_Mac/r8.1.0/BuildTools/VerifyTCSig
#                Revision 4, 8/16/18 - 4.0d1
SymantecVerifyProgramPath="VerifyTCSig/4.0d1/$SymantecVerifyProgramName"
SymantecVerifyProgramPathNLOK="VerifyTCSig/NLOK.0d1/$SymantecVerifyProgramName"
SDSBundle="/Library/Application Support/Symantec/Silo/MES/Daemon/PlugIns/SDS.bundle"
NortonFontUIBundle="/Library/Application Support/Symantec/Silo/MES/CoreUI/NortonUIFonts.bundle"
VerifyTCSigBypassPatternKey="SkipVerifyTCSigForPathsThatMatchPattern"

# *** Function Declarations ***

DefineCurrentUserDesktopAndOS()
{
   # Usage:     DefineCurrentUserDesktopAndOS [-d] [-o] [-u] [exit_code]
   #
   # Version:   1.0.2
   #
   # Summary:   Assigns values to the following variables:
   #               macOSVersion
   #               macOSVersionNumeric
   #               UserDesktop
   #               UserHomeDir
   #               UserThatLoggedIntoComputer
   #
   #            macOSVersionNumeric = major * 1,000,000 + minor * 1,000 + bug
   #
   #            Example: If macOSVersion is 10.14.6, macOSVersionNumeric would
   #            be 10014006.
   #
   #            If one of them could not be defined, login window is active,
   #            or UserDesktop is not a directory, then an error message is
   #            shown and this script is exited with exit_code or with 78 if
   #            exit_code was not passed.
   #
   # Options:
   #    -d      Don't show error and don't exit if Desktop is not a directory.
   #    -o      Don't show error and don't exit if OS version could not be
   #            determined.
   #    -u      Don't show error and don't exit if login window is active or
   #            if active user could not be determined.
   #    exit_code
   #            The exit code to use when an error occurs. The default is 78.
   #            Exit code must be an integer in range 0-255.
   #
   # History: 1.0.1 - 08/07/2014 - Corey Swertfager:
   #                  * Revised the way the user's home directory is determined.
   #                  * Added definition of UserHomeDir.
   #                  * Added exit_code option.
   #                  * The -d option no longer prevents exiting with an error
   #                    when login window is active or when active user could
   #                    not be determined; use the -u to prevent that.
   #          1.0.2 - 07/28/2020 - Corey Swertfager:
   #                  * Added macOS 11+ support.
   #
   local DoShowDesktopError=true
   local DoShowLoginWindowActiveOrNoActiveUserError=true
   local DoShowOSVersionError=true
   local ErrorString=""
   local ExitValue=78
   local macOSVersionBug
   local macOSVersionMajor
   local macOSVersionMinor
   while [ $# -gt 0 ] ; do
      case "$1" in
         -d)
            DoShowDesktopError=false
            ;;
         -o)
            DoShowOSVersionError=false
            ;;
         -u)
            DoShowLoginWindowActiveOrNoActiveUserError=false
            ;;
         *)
            ExitValue="$1"
            ;;
      esac
      shift
   done
   macOSVersion=`cat /System/Library/CoreServices/SystemVersion.plist 2>/dev/null | tr '\015' '\012' | grep . | grep -A1 '<key>ProductVersion<' | tail -n 1 | awk -F '<string>' '{print $2}' | awk -F '<' '{print $1}'`
   # If an error occurred or OS version does not begin with a digit
   if [ $? != 0 -o -z "`printf "%s" "$macOSVersion" | grep '^[0-9]'`" ] ; then
      macOSVersion=""
      macOSVersionNumeric=""
   else
      macOSVersionMajor=`printf "%s" "$macOSVersion" | awk -F . '{print $1}'`
      macOSVersionMinor=`printf "%s" "$macOSVersion" | awk -F . '{print $2}'`
      [ -z "$macOSVersionMinor" ] && macOSVersionMinor=0
      macOSVersionBug=`printf "%s" "$macOSVersion" | awk -F . '{print $3}'`
      [ -z "$macOSVersionBug" ] && macOSVersionBug=0
      macOSVersionNumeric=$(( macOSVersionMajor * 1000000 + macOSVersionMinor * 1000 + macOSVersionBug ))
   fi
   UserDesktop=""
   UserHomeDir=""
   UserThatLoggedIntoComputer=`printf "open\nshow State:/Users/ConsoleUser\nq\n" | scutil | tr '\015' '\012' | awk '{print $1$2$3}' | grep 'Name:.' | tail -n 1 | awk -F : '{print $2}'`
   if [ "z$UserThatLoggedIntoComputer" = zloginwindow ] ; then
      $DoShowLoginWindowActiveOrNoActiveUserError && ErrorString="WARNING: The login window is active."
   elif [ -z "$UserThatLoggedIntoComputer" ] ; then
      $DoShowLoginWindowActiveOrNoActiveUserError && ErrorString="WARNING: There is no user logged in or active user could not be determined."
   else
      UserHomeDir=`echo $(eval echo ~"$UserThatLoggedIntoComputer")`
      # If home directory is not a directory or could not be evaluated
      if [ ! -d "$UserHomeDir" -o -n "`printf "%s" "$UserHomeDir" | grep '^~'`" ] ; then
         UserHomeDir=""
         $DoShowDesktopError && ErrorString="$ErrorString
WARNING: Could not determine home directory for user $UserThatLoggedIntoComputer."
      else
         UserDesktop="$UserHomeDir/Desktop"
         if [ -f "$UserDesktop" ] ; then
            $DoShowDesktopError && ErrorString="$ErrorString
WARNING: $UserDesktop is a file instead of a directory."
         elif [ ! -d "$UserDesktop" ] ; then
            $DoShowDesktopError && ErrorString="$ErrorString
WARNING: User Desktop folder ($UserDesktop) could not be found."
         fi
      fi
   fi
   # If numeric OS version could not be determined
   if [ -z "$macOSVersionNumeric" ] ; then
      $DoShowOSVersionError && ErrorString="$ErrorString
WARNING: Mac OS version could not be determined."
   fi
   if [ "$ErrorString" ] ; then
      echo "$ErrorString" | grep .
      echo "         `basename "$0"` aborted."
      [ "`ExitScript "$ExitValue" 2>/dev/null`" ] && ExitScript "$ExitValue" || exit "$ExitValue"
   fi
}

ExitScript()
{
   # Usage:     ExitScript [$1 [$2]]
   # Arguments: $1 = The value to pass when calling the exit command.
   #            $2 = Error string to show before exiting script.
   # Summary:   Checks to see if ShowQuitMessage and RunScriptAsStandAlone
   #            variables are set to true. If so, a message is displayed;
   #            otherwise, no message is displayed. The script is then
   #            exited and passes $1 to exit command. If nothing is passed
   #            to $1, then 0 is passed to exit command. If a non-integer
   #            is passed to $1, then 255 is passed to exit command.
   #            If $2 is passed, then it is echoed before exiting and is padded
   #            by blank lines if $2 is not "".
   #
   rm -f "$TempFile" 2>/dev/null
   if [ $# -gt 1 ] ; then
      if [ "$2" ] ; then
         echo
         echo "$2"
      fi
      echo
   fi
   if [ "z$ShowQuitMessage" = ztrue -a "z$RunScriptAsStandAlone" = ztrue ] ; then
      [ $# -lt 2 ] && echo
      echo "NOTE: If you double-clicked this script, quit Terminal application now."
      echo
   fi
   [ -z "$1" ] && exit 0
   [ -z "`expr "$1" / 1 2>/dev/null`" ] && exit 78 || exit $1
}

GetAdminPassword()
{
   # Usage:     GetAdminPassword [$1]
   # Argument:  $1 - Prompt for password. If true is passed, a user that
   #                 is not root will always be asked for a password. If
   #                 something other than true is passed or if nothing is
   #                 passed, then a user that is not root will only be
   #                 prompted for a password if authentication has lapsed.
   # Summary:   Gets an admin user password from the user so that
   #            future sudo commands can be run without a password
   #            prompt. The script is exited with a value of 1 if
   #            the user enters an invalid password or if the user
   #            is not an admin user. If the user is the root user,
   #            then there is no prompt for a password (there is
   #            no need for a password when user is root).
   #            NOTE: Make sure ExitScript function is in the script.
   #
   # If root user, no need to prompt for password
   [ "`whoami`" = "root" ] && return 0
   echo
   # If prompt for password
   if [ "$1" = "true" -o "$1" = "true" ] ; then
      sudo -k   # Make sudo require a password the next time it is run
      echo "You must be an admin user to run this script."
   fi
   # A dummy sudo command to get password
   sudo -p "Please enter your admin password: " date 2>/dev/null 1>&2
   if [ ! $? = 0 ] ; then       # If failed to get password, alert user and exit script
      echo "You entered an invalid password or you are not an admin user. Script aborted."
      ExitScript 1
   fi
}

ProcessArguments()
{
   # Usage:     ProcessArguments [ --OptionTakesUnparsedArgument=string ] [ --OptionIsOneArgument=string ] "$@"
   #
   # Version:   1.0.1
   #
   # Summary:   Processes arguments passed to script. Arguments beginning with a
   #            single hyphen (-) are parsed into separate options except when an
   #            argument is negative integer. Arguments beginning with two hypens
   #            are treated as one argument; if the argument contains is an equals
   #            sign (=), the string after the first "=" is treated as a separate
   #            argument (i.e., the value assigned to the double-hyphen argument).
   #
   #            For each --OptionTakesUnparsedArgument passed before "$@", the string
   #            after "=" is used as an option that takes the next argument in full
   #            without parsing it (see examples below); string must be a hyphen
   #            followed by a single character.
   #
   #            For each --OptionIsOneArgument passed before "$@", the string after
   #            "=" is used as an option that should be treated as a single argument.
   #            This is useful when processing an argument that begins with a single
   #            hyphen to avoid having that argument parsed into separate options.
   #            The referenced option cannot be embedded within other options (see
   #            final example below).
   #
   #            "$@" must be the last argument passed to ProcessArguments. Put all custom
   #            option handling between "--- Customized argument handling begins here ---"
   #            and "--- Customized argument handling ends here ---".
   #
   # Note:      ProcessArgumentsNextArgument may be called to verify and obtain the
   #            next argument after or before a given option; see that function's usage
   #            for more details. OriginalArgumentNumber can be used to determine if
   #            two arguments were originally passed within the same string of options.
   #
   # Examples:  These examples have expanded the arguments passed as "$@".
   #
   #            ProcessArguments -ab -c
   #               Would process three arguments: -a, -b, and -c
   #            ProcessArguments --ab -c
   #               Would process two arguments: --ab and -c
   #            ProcessArguments --equation=a=b+c
   #               Would process two arguments: --equation and a=b+c
   #            ProcessArguments -10
   #               Would process one argument: -10
   #            ProcessArguments -10a
   #               Would process three arguments: -1, -0, -a
   #            ProcessArguments --OptionTakesUnparsedArgument=-e -e -ger
   #               Would process two arguments: -e and -ger
   #            ProcessArguments --OptionTakesUnparsedArgument=-e -peer
   #               Would process three arguments: -p, -e, and er
   #            ProcessArguments --OptionTakesUnparsedArgument=-e --OptionTakesUnparsedArgument=-t -eter -ter
   #               Would process four arguments: -e, ter, -t, and er
   #            ProcessArguments --OptionIsOneArgument=-hi -hi
   #               Would process one argument: -hi
   #            ProcessArguments --OptionIsOneArgument=-hi -his
   #               Would process three arguments: -h, -i, and -s
   #
   # History: 1.0.1 - 06/23/2013 - Corey Swertfager:
   #                  * Added processing of options within a string that begins
   #                    with a single hyphen.
   #                  * Added --OptionTakesUnparsedArgument option.
   #                  * Added --OptionIsOneArgument option.
   #
   local ArgList=""
   local ArgsToAdd
   local ArgWasAdded=false
   local CurrentArgNumber=1
   local CurrentArgument
   local CurrentCharacter
   local DoNotParseNextArgument=false
   local NextArgument=""
   local NumberOfArgumentsPassed
   local NumberOfArgumentsToUse=0
   local OptionToAdd
   local OriginalArgumentNumber=0
   local OriginalArgumentNumberList=""
   local RemainingOptionsInString
   local TableOfOptionsWithoutParsing="*** Each option in this table will have its succeeding argument left unparsed. ***"
   local TableOfUndividedArguments="*** Each item in this table should each be treated as single argument. ***"
   while [ "$1" ] ; do
      case "$1" in
         --OptionIsOneArgument)
            ExitScript 99 "WARNING: Bad use of --OptionIsOneArgument passed to ProcessArguments:
         \"$1\""
            ;;
         --OptionIsOneArgument=*)
            OptionToAdd=`printf "%s" "$1" | awk '{match($0,"=") ; print substr($0,RSTART+1)}'`
            [ -z "$OptionToAdd" ] && ExitScript 99 "WARNING: Bad use of --OptionIsOneArgument passed to ProcessArguments:
         \"$1\""
            TableOfUndividedArguments="$TableOfUndividedArguments
$OptionToAdd"
            ;;
         --OptionTakesUnparsedArgument|--OptionTakesUnparsedArgument=*)
            OptionToAdd=`printf "%s" "$1" | awk '{match($0,"=") ; print substr($0,RSTART+1)}'`
            [ -z "`printf "%s" "$OptionToAdd" | grep -xe '-.'`" ] && ExitScript 99 "WARNING: Bad use of --OptionTakesUnparsedArgument passed to ProcessArguments:
         \"$1\""
            TableOfOptionsWithoutParsing="$TableOfOptionsWithoutParsing
$OptionToAdd"
            ;;
         *)
            break
            ;;
      esac
      shift
   done
   NumberOfArgumentsPassed=$#
   while [ $# != 0 ] ; do
      let OriginalArgumentNumber=$OriginalArgumentNumber+1
      # If argument is in the list of arguments whose next argument should not be parsed
      if [ "`printf "%s" "$1" | grep -xF "$TableOfOptionsWithoutParsing"`" ] ; then
         ArgsToAdd="$1"
         OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
         DoNotParseNextArgument=true
      # Else if argument is in the list of arguments that should be treated as one argument
      elif [ "`printf "%s" "$1" | grep -xF "$TableOfUndividedArguments"`" ] ; then
         ArgsToAdd="$1"
         OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
      else
         case "$1" in
            -|-?)
               # If argument was a hyphen or a hyphen followed by a single character
               ArgsToAdd="$1"
               OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
               DoNotParseNextArgument=false
               ;;
            --*)
               # If a value was passed to the option that begins with --
               if [ "`printf "%s" "$1" | grep =`" ] ; then
                  # Add the option and its value as separate arguments
                  ArgsToAdd="`printf "%s" "$1" | awk -F = '{print $1}'`
`printf "%s" "$1" | awk '{match($0,"=") ; print substr($0,RSTART+1)}'`"
                  OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber
$OriginalArgumentNumber"
               else
                  ArgsToAdd="$1"
                  OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
               fi
               DoNotParseNextArgument=false
               ;;
            -*)
               # If argument should not be parsed or is a negative integer
               if [ $DoNotParseNextArgument = true -o -z "`printf "%s" "$1" | awk '{print substr($0,2)}' | tr -d '[:digit:]'`" ] ; then
                  # Treat argument as a single argument
                  ArgsToAdd="$1"
                  OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
                  DoNotParseNextArgument=false
               else
                  # Parse string into separate arguments
                  ArgsToAdd=""
                  RemainingOptionsInString=`printf "%s" "$1" | awk '{print substr($0,2)}'`
                  while [ "$RemainingOptionsInString" ] ; do
                     CurrentCharacter=`printf "%s" "$RemainingOptionsInString" | awk '{print substr($0,1,1)}'`
                     # Prefix the character with a hyphen and add as an argument
                     if [ "$ArgsToAdd" ] ; then
                        ArgsToAdd="$ArgsToAdd
-$CurrentCharacter"
                     else
                        ArgsToAdd="-$CurrentCharacter"
                     fi
                     OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
                     RemainingOptionsInString=`printf "%s" "$RemainingOptionsInString" | awk '{print substr($0,2)}'`
                     # If this is an option whose next string should not be parsed
                     if [ "`printf "%s" "$TableOfOptionsWithoutParsing" | grep -xe "-$CurrentCharacter"`" ] ; then
                        # If string has characters remaining after that option
                        if [ "$RemainingOptionsInString" ] ; then
                           # Add remainder of string as the unparsed string argument
                           ArgsToAdd="$ArgsToAdd
$RemainingOptionsInString"
                           OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
                        else
                           # Use next argument passed as unparsed string argument
                           DoNotParseNextArgument=true
                        fi
                        break
                     fi
                  done
               fi
               ;;
            *)
               ArgsToAdd="$1"
               OriginalArgumentNumberList="$OriginalArgumentNumberList
$OriginalArgumentNumber"
               DoNotParseNextArgument=false
               ;;
         esac
      fi
      if $ArgWasAdded ; then
         ArgList="$ArgList
$ArgsToAdd"
      else
         ArgList="$ArgsToAdd"
      fi
      ArgWasAdded=true
      shift
   done
   if [ $NumberOfArgumentsPassed -gt 0 ] ; then
      # Add a non-blank line to ArgList in case last argument passed was ""
      ArgList="$ArgList
           TheEnd"
      OriginalArgumentNumberList=`echo "$OriginalArgumentNumberList" | grep .`
      NumberOfArgumentsToUse=`printf "%s" "$ArgList" | grep "" -c`
      let NumberOfArgumentsToUse=$NumberOfArgumentsToUse-1
   fi
   # --- Customized argument handling begins here ---
   CheckAppleSign=true
   CheckSymantecSign=true
   DoReportEnvelopeErrors=false
   ErrorsOnly=true
   FileToUse=""
   InstalledFilesOnly=true
   ListOfFolders=""
   NumberOfProcessArgumentsLeft=$#
   ScriptExitCode=0
   SortInputFile=true
   while [ $CurrentArgNumber -le $NumberOfArgumentsToUse ] ; do
      CurrentArgument=`printf "%s" "$ArgList" | head -n $CurrentArgNumber | tail -n 1`
      OriginalArgumentNumber=`echo "$OriginalArgumentNumberList" | head -n $CurrentArgNumber | tail -n 1`
      case "$CurrentArgument" in
         --SEPFerrari)
            FileToUse="$FileListsDir/$SEPFerrariList"
            ;;
         --SEPTesla)
            FileToUse="$FileListsDir/$SEPTeslaList"
            ;;
         -a)
            CheckAppleSign=true
            CheckSymantecSign=false
            ;;
         -e)
            ErrorsOnly=true
            ;;
         -f)
            ErrorsOnly=false
            ;;
         -H)
            ShowUsage 0
            ;;
         -h)
            ShowHelp 0
            ;;
         -I)
            InstalledFilesOnly=false
            ;;
         -i)
            InstalledFilesOnly=true
            ;;
         -r)
            DoReportEnvelopeErrors=true
            ;;
         -s)
            CheckAppleSign=false
            CheckSymantecSign=true
            ;;
         -u)
            SortInputFile=false
            ;;
         -V)
            echo $Version
            exit 0
            ;;
         "")
            :
            ;;
         *)
            if [ -d "$CurrentArgument" ] ; then
               ListOfFolders=`ShowFullFilePath -P "$CurrentArgument"`
            elif [ -f "$CurrentArgument" ] ; then
               FileToUse=`ShowFullFilePath -P "$CurrentArgument"`
            else
               ShowUsage 78 "ERROR: Invalid option or file: \"$CurrentArgument\"."
            fi
            ;;
      esac
      let CurrentArgNumber=$CurrentArgNumber+1
   done
   if [ -z "$ListOfFolders" ] ; then
      ListOfFolders=/
   fi
   # If OS version is less than 10.5
   if [ $macOSVersionNumeric -lt 10005000 ] ; then
      if $CheckAppleSign ; then
         ! $CheckSymantecSign && ExitScript 78 "NOTE: Cannot verify Apple signatures unless running OS 10.5 or later."
         CheckAppleSign=false
      fi
   fi
   # --- Customized argument handling ends here ---
}

ProcessArgumentsNextArgument()
{
   # Usage:     ProcessArgumentsNextArgument [exit_code] [-F | -P] [-p | -r ] [operator]
   #
   # Version:   1.0.0
   #
   # Arguments: exit_code   Pass integer in range 0-255 to ShowUsage when next
   #                        argument is missing or invalid. If exit_code is not
   #                        specified, 0 is assumed.
   #
   #            -F          Assign the full logical path to NextArgumentFullPath.
   #                        This is the default. ShowFullFilePath function must
   #                        be included in script. If no operator was passed, -E
   #                        is the assumed operator.
   #
   #            -P          Assign the full physical path to NextArgumentFullPath.
   #                        ShowFullFilePath function must be included in script.
   #                        If no operator was passed, -e is the assumed operator.
   #
   #            -p          Get previous argument instead of next argument. If
   #                        there is no previous argument, sets NextArgument to ""
   #                        and returns 1.
   #
   #            -r          Return 1 instead of exiting script if there is no next
   #                        argument. Sets NextArgument to "".
   #
   #            operator    Operator used to test next argument:
   #                           -d   Folder
   #                           -E   File, folder, or link
   #                           -e   File, folder, or link to an existing file/folder
   #                           -f   File
   #                           -i [min [max]]
   #                                Integer in range min-max; pass "" to min and
   #                                an integer to max if there is no minimum but
   #                                a maximum is desired; pass "" to to both min
   #                                and max if passing another option after the -i
   #                                option. Also tests to see if the value of the
   #                                next argument is out of range for the currently
   #                                running version of Mac OS.
   #                           -L   Link; does not check to see if link is broken
   #                                unless -P option was also passed
   #                           -l   Link to an existing file/folder
   #                           -n   Non-null string
   #
   # Summary:   Called by ProcessArguments 1.0.1 or later to assign values to:
   #
   #               CurrentArgNumber
   #               NextArgument
   #               NextArgumentFullPath
   #               NextArgumentOriginalArgumentNumber
   #
   #            using pre-existing values of:
   #
   #               ArgList
   #               CurrentArgNumber
   #               CurrentArgument
   #               OriginalArgumentNumberList
   #               NumberOfArgumentsToUse
   #
   #            Returns 0 if next or previous argument was assigned to NextArgument,
   #            CurrentArgNumber was incremented (or decremented if -p was passed),
   #            and NextArgumentOriginalArgumentNumber was assigned.
   #
   #            Assigns "" to NextArgument and NextArgumentFullPath and returns 1 if
   #            -p is passed and there is no previous argument, or if -r is passed
   #            and there is no next argument; otherwise, calls ShowUsage to show
   #            error message and exit script if operator test fails or if there is
   #            no next or previous argument.
   #
   # Note:      ShowFullFilePath function must be included in script in order to
   #            assign a value to NextArgumentFullPath.
   #
   # Examples:  ProcessArgumentsNextArgument
   #               Returns 0 if there was a next argument; otherwise, passes 0 to
   #               ShowUsage and displays error message about missing argument.
   #            ProcessArgumentsNextArgument -r
   #               Returns 0 if there was a next argument; otherwise, assigns "" to
   #               NextArgument and NextArgumentFullPath, then returns 1.
   #            ProcessArgumentsNextArgument 2 -d
   #               Returns 0 if NextArgument was set to a folder; otherwise, passes
   #               2 to ShowUsage and displays error message if the next argument is
   #               missing or is not a folder.
   #            ProcessArgumentsNextArgument 3 -r -d
   #               Returns 0 if NextArgument was set to a folder. If the next argument
   #               is missing, assigns "" to NextArgument and NextArgumentFullPath,
   #               then returns 1. If next argument is not a folder, passes 3 to
   #               ShowUsage and displays error message.
   #            ProcessArgumentsNextArgument 4 -i 1
   #               Returns 0 if NextArgument was set to an integer; otherwise, passes
   #               4 to ShowUsage and displays error message if the next argument is
   #               missing, is not an integer, or is less than 1.
   #            ProcessArgumentsNextArgument -i "" 100 5
   #               Returns 0 if NextArgument was set to an integer; otherwise, passes
   #               5 to ShowUsage and displays error message if the next argument is
   #               missing, is not an integer, or is greater than 100.
   #            ProcessArgumentsNextArgument -i "" "" 6
   #               Returns 0 if NextArgument was set to an integer; otherwise, passes
   #               6 to ShowUsage and displays error message if the next argument is
   #               missing or is not an integer.
   #
   local DoShowMissingError=true
   local DirectionText=after
   local ExitCode=0
   local ErrorText=""
   local GoToPreviousArgument=false
   local Max
   local Min
   local NextArgumentOriginal
   local PathOption=""
   local TestOperator=""
   NextArgumentFullPath=""
   while [ "$1" ] ; do
      case "$1" in
         -d)
            ErrorText="folder"
            TestOperator="$1"
            ;;
         -E)
            ErrorText="file, folder, or link"
            TestOperator="$1"
            ;;
         -e)
            ErrorText="file or folder"
            TestOperator="$1"
            ;;
         -F)
            PathOption="$1"
            if [ -z "$ErrorText" ] ; then
               ErrorText="file, folder, or link"
               TestOperator="-E"
            fi
            ;;
         -f)
            ErrorText="file"
            TestOperator="$1"
            ;;
         -i)
            ErrorText="integer"
            TestOperator="$1"
            Min="$2"
            Max="$3"
            shift 2
            ;;
         -L)
            ErrorText="link"
            TestOperator="$1"
            [ "z$PathOption" = "z-P" ] && ErrorText="unbroken link"
            ;;
         -l)
            ErrorText="unbroken link"
            TestOperator="-L"
            ;;
         -n)
            ErrorText="non-null string"
            TestOperator="$1"
            ;;
         -P)
            PathOption="$1"
            if [ -z "$ErrorText" ] ; then
               ErrorText="file or folder"
               TestOperator="-e"
            elif [ "$ErrorText" = "link" ] ; then
               ErrorText="unbroken link"
            fi
            ;;
         -p)
            GoToPreviousArgument=true
            DirectionText=before
            ;;
         -r)
            DoShowMissingError=false
            ;;
         *)
            ExitCode=`printf "%s" "$1" | tr -d -c "[:digit:]"`
            [ -z "$ExitCode" ] && ExitCode=0
            ;;
      esac
      shift
   done
   if $GoToPreviousArgument ; then
      if [ $CurrentArgNumber -gt 1 ] ; then
         let CurrentArgNumber=$CurrentArgNumber-1
         NextArgument=`printf "%s" "$ArgList" | head -n $CurrentArgNumber | tail -n 1`
         NextArgumentOriginalArgumentNumber=`echo "$OriginalArgumentNumberList" | head -n $CurrentArgNumber | tail -n 1`
      else
         NextArgument=""
         NextArgumentFullPath=""
         return 1
      fi
   # Else if there are no more arguments in ArgList
   elif [ $CurrentArgNumber = $NumberOfArgumentsToUse ] ; then
      if $DoShowMissingError ; then
         ShowUsage $ExitCode "ERROR: Nothing was passed after $CurrentArgument" >&2
      else
         NextArgument=""
         NextArgumentFullPath=""
         return 1
      fi
   else
      let CurrentArgNumber=$CurrentArgNumber+1
      NextArgument=`printf "%s" "$ArgList" | head -n $CurrentArgNumber | tail -n 1`
      NextArgumentOriginalArgumentNumber=`echo "$OriginalArgumentNumberList" | head -n $CurrentArgNumber | tail -n 1`
   fi
   NextArgumentFullPath=`ShowFullFilePath $PathOption "$NextArgument"`
   if [ "z$ErrorText" = zinteger ] ; then
      NextArgumentOriginal="$NextArgument"
      if [ -z "$NextArgument" ] ; then
         ShowUsage $ExitCode "ERROR: Argument passed $DirectionText $CurrentArgument is not an integer: \"$NextArgumentOriginal\"" >&2
      # Else if argument contains something other than a hyphen or digits
      elif [ "`printf "%s" "$NextArgument" | tr -d "[:digit:]-"`" ] ; then
         ShowUsage $ExitCode "ERROR: Argument passed $DirectionText $CurrentArgument is not an integer:
       \"$NextArgumentOriginal\"" >&2
      # Else if argument contains a hyphen that is not at the beginning
      elif [ "`printf "%s" "$NextArgument" | grep '..*-'`" ] ; then
         ShowUsage $ExitCode "ERROR: Argument passed $DirectionText $CurrentArgument is not an integer:
       \"$NextArgumentOriginal\"" >&2
      fi
      NextArgument=`expr "$NextArgument" / 1 2>/dev/null`
      test "$NextArgumentOriginal" -eq "$NextArgument" 2>/dev/null
      if [ $? != 0 ] ; then
         ShowUsage $ExitCode "ERROR: Value passed $DirectionText $CurrentArgument is out of range for this OS:
       $NextArgumentOriginal" >&2
      fi
      # If minimum value was specified
      if [ "$Min" ] ; then
         [ $NextArgument -lt $Min ] && ShowUsage $ExitCode "ERROR: Value passed $DirectionText $CurrentArgument ($NextArgumentOriginal) is less than
       minimum value ($Min)." >&2
      fi
      # If maximum value was specified
      if [ "$Max" ] ; then
         [ $NextArgument -gt $Max ] && ShowUsage $ExitCode "ERROR: Value passed $DirectionText $CurrentArgument ($NextArgumentOriginal) is greater than
       maximum value ($Max)." >&2
      fi
   elif [ "z$ErrorText" = "zfile, folder, or link" ] ; then
      [ ! -e "$NextArgument" -a ! -L "$NextArgument" ] && ShowUsage $ExitCode "ERROR: Argument passed $DirectionText $CurrentArgument is not a $ErrorText:
       \"$NextArgument\"" >&2
   elif [ "z$ErrorText" = "zunbroken link" ] ; then
      if [ ! -L "$NextArgument" ] ; then
         ShowUsage $ExitCode "ERROR: Argument passed $DirectionText $CurrentArgument is not a link:
       \"$NextArgument\"" >&2
      # Else if link is broken
      elif [ ! -e "$NextArgument" ] ; then
         ShowUsage $ExitCode "ERROR: The target of the link passed $DirectionText $CurrentArgument does not exist:
       \"$NextArgument\"" >&2
      fi
   elif [ "$ErrorText" ] ; then
      [ ! $TestOperator "$NextArgument" ] && ShowUsage $ExitCode "ERROR: Argument passed $DirectionText $CurrentArgument is not a $ErrorText:
       \"$NextArgument\"" >&2
   fi
   if [ "$PathOption" ] ; then
      if [ -z "$NextArgumentFullPath" ] ; then
         if [ -L "$NextArgument" ] ; then
            ShowUsage $ExitCode "ERROR: The target of the link passed $DirectionText $CurrentArgument does not exist:
       \"$NextArgument\"" >&2
         else
            ExitScript $ExitCode "WARNING: ShowFullFilePath function could not resolve path for:
         \"$NextArgument\"" >&2
         fi
      fi
   fi
   return 0
}

ShowContents()
{
   # Usage1:    ShowContents [-c] [-w] File [TextToShow]
   # Usage2:    ShowContents [-c] [-w] -s String [TextToShow]
   # Summary:   Displays contents of File or String. If there are more than
   #            $LINES or 23 lines, more command is used, using TextToShow as
   #            the name of the file; if TextToShow is not passed, "....." is
   #            used. If -c is specified, screen is cleared beforehand.
   #            If -w is specified, then width of strings will be factored
   #            into the line count (this option makes output slower when
   #            the number of lines is less than $LINES or 23).
   #
   local SCLineCount
   local SCCurrentDir
   local SCTempFolder
   local SCTempFile
   local SCColumns
   local SCColumnsMax
   local SCColumnsMaxDefault=80
   local SCColumnsRemainder
   local CSDoAdjustForWidth=false
   local CSDoUseString=false
   local SCEachLine
   local SCGrepPattern='.'
   local SCLineFactor
   local SCLines
   local SCLinesMax
   local SCLinesMaxDefault=23
   local SCText
   while [ "$1" ] ; do
      if [ "z$1" = z-c ] ; then
         clear
      elif [ "z$1" = z-s ] ; then
         CSDoUseString=true
      elif [ "z$1" = z-w ] ; then
         CSDoAdjustForWidth=true
      else
         break
      fi
      shift
   done
   [ "$COLUMNS" ] && SCColumnsMax=`expr "$COLUMNS" - 0 2>/dev/null`
   [ -z "$SCColumnsMax" ] && SCColumnsMax=$SCColumnsMaxDefault
   [ "$LINES" ] && SCLinesMax=`expr "$LINES" - 1 2>/dev/null`
   [ -z "$SCLinesMax" ] && SCLinesMax=$SCLinesMaxDefault
   [ $SCColumnsMax -ge $SCColumnsMaxDefault ] && SCGrepPattern='.................................................................................'
   if $CSDoUseString ; then
      SCLineCount=`printf "%s\n" "$1" | grep -c ""`
      $CSDoAdjustForWidth && SCText=`printf "%s\n" "$1" | grep "$SCGrepPattern"`
   elif [ -f "$1" ] ; then
      SCLineCount=`grep -c "" "$1"`
      $CSDoAdjustForWidth && SCText=`grep "$SCGrepPattern" "$1"`
   else
      return 1
   fi
   if $CSDoAdjustForWidth ; then
      if [ $SCLineCount -le $SCLinesMax ] ; then
         IFS='
'
         for SCColumns in `printf "%s" "$SCText" | awk '{print length($0)}'` ; do
            [ $SCLineCount -gt $SCLinesMax ] && break
            SCLineFactor=`expr $SCColumns / $SCColumnsMax`
            [ `expr $SCColumns % $SCColumnsMax` -gt 0 ] && let SCLineFactor=$SCLineFactor+1
            [ $SCLineFactor -gt 1 ] && let SCLineCount=$SCLineCount+$SCLineFactor-1
         done
      fi
   fi
   if $CSDoUseString ; then
      if [ $SCLineCount -gt $SCLinesMax ] ; then
         SCCurrentDir=`pwd`
         SCTempFolder="/private/tmp/$FullScriptName-SC-`date +"%Y%m%d%H%M%S"`"
         mkdir "$SCTempFolder" 2>/dev/null
         [ ! -d "$SCTempFolder" ] && return 1
         cd "$SCTempFolder" 2>/dev/null
         [ "$2" ] && SCTempFile="$2" || SCTempFile="....."
         printf "%s\n" "$1" >"$SCTempFile"
         more -E "$SCTempFile"
         cd "$SCCurrentDir" 2>/dev/null
         rm -rf "$SCTempFolder" 2>/dev/null
      else
         printf "%s\n" "$1"
      fi
   elif [ -f "$1" ] ; then
      if [ $SCLineCount -gt $SCLinesMax ] ; then
         SCCurrentDir=`pwd`
         SCTempFolder="/private/tmp/$FullScriptName-SC-`date +"%Y%m%d%H%M%S"`"
         mkdir "$SCTempFolder" 2>/dev/null
         [ ! -d "$SCTempFolder" ] && return 1
         [ "$2" ] && SCTempFile="$2" || SCTempFile="....."
         cat "$1" >"$SCTempFolder/$SCTempFile"
         cd "$SCTempFolder" 2>/dev/null
         more -E "$SCTempFile"
         cd "$SCCurrentDir" 2>/dev/null
         rm -rf "$SCTempFolder" 2>/dev/null
      else
         cat "$1"
      fi
   fi
   return 0
}

ShowFullFilePath()
{
   # Usage:     ShowFullFilePath [-a] [-P | -L] [-e] Path [[-e] Path]
   # Version:   1.0.2
   # Summary:   Prints the full path starting at / of Path if Path exists
   #            and Path is accessible by the user calling this function.
   #            Run this function as root to ensure full path displaying.
   #            If there is more than one existing file that matches the
   #            name, then only the first path that the shell matches is
   #            printed unless -a or more than one path is specified.
   #            You can specify wild card characters ? and * and other
   #            argument operators in the Path (e.g., "../*", "a?.txt",
   #            "[ab]*").
   # Options:   -a   Show all matching paths, sorted alphanumerically. If
   #                 -P is not passed, the same file may be shown multiple
   #                 times if there is more than one matching link that
   #                 points to it.
   #            -e <Path>
   #                 Treat argument after -e as a path. Use -e to treat
   #                 -a, -e, -L, or -P as a path.
   #            -L   Show logical path, even if a file pointed to by a link
   #                 doesn't exist. This is the default.
   #            -P   Show physical path. If a link points to a file that
   #                 does not exist, the path won't be shown.
   # History: 1.0.1 - Added -e option and ability to pass multiple paths.
   #                  Arguments can now be passed in any order.
   #                  Fixed error that could occur when resolving links
   #                  with long paths.
   #          1.0.2 - Modified for case-sensitive volume compatibility.
   #                  Made temporary file names more distinctive.
   #
   local SFFPArgCount=$#
   local SFFPCurrentDir
   local SFFPCurrentDirTranslated
   local SFFPEachLine
   local SFFPEachPath
   local SFFPFile
   local SFFPLDir
   local SFFPLLinkLS
   local SFFPLLinkPath
   local SFFPLPath
   local SFFPPathOption=-L
   local SFFPSaveIFS="$IFS"
   local SFFPShowAll=false
   local SFFPTempBase=/private/tmp/ShowFullFilePath-`/usr/bin/basename "$0"`-`/bin/date +"%Y%m%d%H%M%S"`
   local SFFPTempFile="$SFFPTempBase.tmp"
   local SFFPTempFile2="$SFFPTempBase-2.tmp"
   /bin/rm -f "$SFFPTempFile" 2>/dev/null
   while [ $SFFPArgCount != 0 ] ; do
      case "$1" in
         -a)
            SFFPShowAll=true
            ;;
         -L|-P)
            SFFPPathOption="$1"
            ;;
         *)
            [ "z$1" = z-e ] && shift
            if [ "$1" ] ; then
               [ -s "$SFFPTempFile" ] && SFFPShowAll=true
               /usr/bin/printf "%s\n" "$1" >>"$SFFPTempFile"
            fi
            ;;
      esac
      shift
      let SFFPArgCount=$SFFPArgCount-1
   done
   [ ! -s "$SFFPTempFile" ] && return
   SFFPCurrentDir=`/bin/pwd`
   SFFPCurrentDirTranslated=`/bin/pwd $SFFPPathOption 2>/dev/null`
   if [ ! -d "$SFFPCurrentDirTranslated" ] ; then
      /bin/rm -f "$SFFPTempFile" 2>/dev/null
      return
   fi
   cd "$SFFPCurrentDirTranslated" 2>/dev/null
   if [ $? != 0 ] ; then
      /bin/rm -f "$SFFPTempFile" 2>/dev/null
      return
   fi
   /usr/bin/printf "" >"$SFFPTempFile2"
   IFS='
'
   for SFFPEachLine in `/bin/cat "$SFFPTempFile" 2>/dev/null` ; do
      cd "$SFFPCurrentDirTranslated" 2>/dev/null
      [ $? != 0 ] && break
      if [ "z$SFFPPathOption" = z-P ] ; then
         SFFPLPath="$SFFPEachLine"
         while [ -L "$SFFPLPath" ] ; do
            [ ! -e "$SFFPLPath" ] && break
            cd "`/usr/bin/dirname "$SFFPLPath" 2>/dev/null`" 2>/dev/null
            [ $? != 0 ] && break
            SFFPLDir=`/bin/pwd -P 2>/dev/null`
            [ ! -d "$SFFPLDir" ] && break
            SFFPLLinkLS=`/bin/ls -ld "$SFFPLPath" 2>/dev/null`
            [ -z "$SFFPLLinkLS" ] && break
            # If link or link target contains " -> " in its name
            if [ "`echo "z$SFFPLLinkLS" | grep ' -> .* -> '`" ] ; then
               SFFPLLinkPath=`/usr/bin/printf "%s" "$SFFPLLinkLS" | /usr/bin/awk -v THESTR="$SFFPLPath -> " '{ match($0,THESTR) ; print substr($0,RSTART+RLENGTH)}'`
            else
               SFFPLLinkPath=`echo "$SFFPLLinkLS" | awk -F " -> " '{print $2}'`
            fi
            # If link target begins with /
            if [ "`/usr/bin/printf "%s" "$SFFPLLinkPath" | grep '^/'`" ] ; then
               SFFPLPath="$SFFPLLinkPath"
            else
               SFFPLPath="$SFFPLDir/$SFFPLLinkPath"
            fi
            [ "`/usr/bin/printf "%s" "$SFFPLPath" | grep '^//'`" ] && SFFPLPath=`echo "$SFFPLPath" | /usr/bin/awk '{print substr($0,2)}'`
         done
         cd "$SFFPCurrentDirTranslated" 2>/dev/null
         [ $? != 0 ] && break
         if [ ! -e "$SFFPLPath" ] ; then
            $SFFPShowAll && continue || break
         fi
         SFFPEachPath="$SFFPLPath"
      else
         SFFPEachPath="$SFFPEachLine"
      fi
      if [ -d "$SFFPEachPath" ] ; then
         cd "$SFFPEachPath" 2>/dev/null
         if [ $? != 0 ] ; then
            $SFFPShowAll && continue || break
         fi
         SFFPFile=""
      elif [ -d "`/usr/bin/dirname "$SFFPEachPath" 2>/dev/null`" ] ; then
         cd "`/usr/bin/dirname "$SFFPEachPath" 2>/dev/null`" 2>/dev/null
         if [ $? != 0 ] ; then
            $SFFPShowAll && continue || break
         fi
         SFFPFile=`basename "$SFFPEachPath" 2>/dev/null`
         [ "z$SFFPFile" = z/ -o "z$SFFPFile" = z. -o "z$SFFPFile" = z.. ] && SFFPFile=""
      elif $SFFPShowAll ; then
         continue
      else
         break
      fi
      SFFPDir=`/bin/pwd $SFFPPathOption 2>/dev/null`
      if [ ! -d "$SFFPDir" ] ; then
         $SFFPShowAll && continue || break
      fi
      SFFPPath="$SFFPDir`[ "z$SFFPFile" != z -a "z$SFFPDir" != z/ -a "z$SFFPDir" != z// ] && echo /`$SFFPFile"
      if [ ! -e "$SFFPPath" -a ! -L "$SFFPPath" ] ; then
         $SFFPShowAll && continue || break
      fi
      [ "`echo "$SFFPPath" | grep '^//'`" ] && SFFPPath=`echo "$SFFPPath" | /usr/bin/awk '{print substr($0,2)}'`
      echo "$SFFPPath" >>"$SFFPTempFile2"
      # If neither option -a nor more than one path was passed, don't show any more names
      ! $SFFPShowAll && break
   done
   IFS=$SFFPSaveIFS
   [ -s "$SFFPTempFile2" ] && /usr/bin/sort -f "$SFFPTempFile2" | /usr/bin/uniq
   /bin/rm -f "$SFFPTempFile" "$SFFPTempFile2" 2>/dev/null
   cd "$SFFPCurrentDir" 2>/dev/null
}

ShowHelp()
{
   # Usage:     ShowHelp [$1]
   # Argument:  $1 = Exit code.
   # Summary:   Displays script usage and help then exits script.
   #            If a number is passed to $1, then script exits with
   #            that number; else, script is not exited.
   #
   ShowContents -s "
Usage1:  $FullScriptName [-e | -f] [-a | -s] [-I | -i] [-r] [-u] [File] [Volume | Folder]
Usage2:  $FullScriptName [-e | -f] [-a | -s] [-I | -i] [-r] [-u] --SEPTesla [Volume | Folder]
Usage3:  $FullScriptName [-e | -f] [-a | -s] [-I | -i] [-r] [-u] --SEPFerrari [Volume | Folder]

Summary: Verifies signatures for files listed in File using $SymantecVerifyProgramName and
         Apple's $AppleVerifyProgramName. If no File is specified, then a master list of
         Symantec files that should be signed will be used. If a volume
         or folder is passed, files in that volume or folder are checked.

NOTE:    $SymantecVerifyProgramName verification is not performed for products that install
         LiveUpdate 6 (i.e., NAV 12, NIS 5).

         $FullScriptName returns the following exit codes:

             0 = no installation errors found
             1 = admin password authentication failed and aborted script
            20 = signature errors were found
            78 = fatal error caused script to exit prematurely

Options:
   --SEPFerrari
         Use $SEPFerrariList file list to verify SEP Ferrari signing.
   --SEPTesla
         Use $SEPTeslaList file list to verify SEP Tesla signing.
   -a    Verify using only $AppleVerifyProgramName.
   -e    Show only errors (i.e., when the verification programs return
         values greater than 0). This is the default output mode.
   -f    Show full output, including output when the verification programs
         return 0. The values returned by those programs are also shown.
   -I    Show errors for files that are not installed.
   -i    Do not show errors for files that are not installed. This is the
         default behavior if -I is not passed.
   -r    Report every \"$ResourceEnvelopeObsoleteError\" error found. If -r is not
         passed, only the total number of those errors is shown.
   -s    Verify using only $SymantecVerifyProgramName. Applicable only if NIS 6 or later
         is installed.
   -u    Leave list of files unsorted. If -u is not specified, files will
         be checked sorted by path. The -u option is useful if you have
         a certain order that you want files to be checked.

Note:    Apple's $AppleVerifyProgramName is available only on OS 10.5 or later, so Apple
         signature checking will be skipped if running earlier OS versions.
"
   [ "$1" ] && exit $1
}

ShowUsage()
{
   # Usage:     ShowUsage [$1 [$2]]
   # Arguments: $1 = Exit code.
   #            $2 = Error message to display before showing usage.
   # Summary:   Displays script usage. If an exit code is passed,
   #            script is exited with that value.
   #
   if [ "$2" ] ; then
      echo
      echo "$2"
      echo
   fi
   ShowHelp | grep "^Usage.*:"
   [ "$2" ] && echo
   [ -n "$1" ] && exit "$1"
}

# *** Beginning of Commands to Execute ***

ScriptPath=`ShowFullFilePath "$0" -P`
[ ! -e "$ScriptPath" ] && ExitScript 78 "WARNING: ScriptPath definition failed. Script aborted."
FullScriptName=`basename "$ScriptPath"`
ScriptDir=`dirname "$ScriptPath"`
SupportDir="$ScriptDir/$SupportDirName"
FileListsDir="$SupportDir/$FileListsDirName"
TempFile="/private/tmp/$FullScriptName-`date +"%Y%m%d%H%M%S"`".tmp
DefineCurrentUserDesktopAndOS
if [ $# -eq 0 ] ; then   # If no arguments were passed to script
   # Run script as if it was double-clicked in Finder so that
   # screen will be cleared and quit message will be displayed.
   RunScriptAsStandAlone=true
else
   # Run script in command line mode so that
   # screen won't be cleared and quit message won't be displayed.
   RunScriptAsStandAlone=false
fi
$RunScriptAsStandAlone && clear
ProcessArguments "$@"
if [ "`whoami`" != root ] ; then
   if $PublicVersion ; then
      GetAdminPassword true   #    Prompt user for admin password
   fi
   # Run this script again as root
   sudo -p "Please enter your admin password: " "$0" "$@"
   ErrorFromSudoCommand=$?
   # If unable to authenticate
   if [ $ErrorFromSudoCommand -eq 1 ] ; then
      echo "You entered an invalid password or you are not an admin user. Script aborted."
      ExitScript 1
   fi
   if $PublicVersion ; then
      sudo -k   # Make sudo require a password the next time it is run
   fi
   exit $ErrorFromSudoCommand #    Exit so script doesn't run again
fi
if [ ! -f "$FileToUse" ] ; then
   if [[ -d "$SDSBundle" ]]; then
        FileToUse="$FileListsDir/$DefaultArmListName"
   else
        FileToUse="$FileListsDir/$DefaultListName"
   fi
   [ ! -f "$FileToUse" ] && FileToUse="$ScriptDir/$DefaultListName"
   DoUseCustomList=false
else
   DoUseCustomList=true
fi
AppleSignErrorCount=0
MissingFileErrorCount=0
NumberOfResourceEnvelopeErrors=0
TCSigErrorCount=0
SomethingShown=false
if $SortInputFile ; then
   ListOfPaths=`cat "$FileToUse" 2>/dev/null | tr '\015' '\012' | grep '^/' | awk -F '#' '{print $1}' | sort -f | uniq`
else
   ListOfPaths=`cat "$FileToUse" 2>/dev/null | tr '\015' '\012' | grep '^/' | awk -F '#' '{print $1}'`
fi
OriginalListOfPaths="$ListOfPaths"
VerifyTCSigBypassPattern=`cat "$FileToUse" 2>/dev/null | tr '\015' '\012' | grep "$VerifyTCSigBypassPatternKey" | awk -F "$VerifyTCSigBypassPatternKey=" '{print $2}'`
IFS='
'
for Folder in $ListOfFolders ; do
   [ "$Folder" = / ] && Folder=""
   NIS5IsInstalled=false
   FirstLiveUpdateDir=`ls -d "$Folder$SiloRootDir/"*"/LiveUpdate" 2>/dev/null | head -n 1`
   if [ -d "$SDSBundle" ] || [ -d "$NortonFontUIBundle" ] ; then
      SymantecVerifyProgramPath="$SymantecVerifyProgramPathNLOK"
   fi
   echo $SymantecVerifyProgramPath
   # If a LiveUpdate directory was found in a Silo
   if [ -d "$FirstLiveUpdateDir" ] ; then
      SymantecVerifyProgram="$SupportDir/$SymantecVerifyProgramPath"
   # Else if LiveUpdate 6 or later is installed
   elif [ -d "$Folder$NewLiveUpdateRegistryFolder" -o -d "$Folder$ActiveLiveUpdateRegistryFolder" ] ; then
      NIS5IsInstalled=true
      SymantecVerifyProgram=""
   fi
   if [ $CheckAppleSign = true -a $DoUseCustomList = false ] ; then
      # Make sure we haven't signed any Apple Swift libraries
      SwiftLibraries=`find "$Folder/Library/Application Support/Symantec" -iname 'libswift*' 2>/dev/null`
      ListOfLibFilesSignedBySymantec=""
      for LibFile in $SwiftLibraries ; do
         if [ "`codesign -vvd "$LibFile" 2>&1 | grep 'Authority=.*Symantec'`" ] ; then
            ListOfLibFilesSignedBySymantec="$ListOfLibFilesSignedBySymantec
$LibFile"
         fi
      done
      if [ "$ListOfLibFilesSignedBySymantec" ] ; then
         ListOfLibFilesSignedBySymantec=`echo "$ListOfLibFilesSignedBySymantec" | grep /`
         NumberOfLibFound=`echo "$ListOfLibFilesSignedBySymantec" | grep -c /`
         let AppleSignErrorCount=$AppleSignErrorCount+$NumberOfLibFound
         echo
         echo "-----------------"
         echo
         if [ $NumberOfLibFound = 1 ] ; then
            echo "ERROR: The following file has been signed using Symantec authority,"
            echo "       but should not have been:"
         else
            echo "ERROR: The following $NumberOfLibFound files have been signed using Symantec authority,"
            echo "       but should not have been:"
         fi
         echo
         echo "$ListOfLibFilesSignedBySymantec"
      fi
   fi
   # If Symantec VerifyTCSig program is not available, don't check Symantec signing
   [ ! -f "$SymantecVerifyProgram" ] && CheckSymantecSign=false
   for EachLine in $ListOfPaths ; do
      PathInFile=`echo "$EachLine" | awk -F "	" '{print $1}'`
      ArgToPassVerifyTCSig=`echo "$EachLine" | awk -F "	" '{print $2}'`
      # If ArgToPassVerifyTCSig is a comment, ignore it
      [ "`printf "%s" "$ArgToPassVerifyTCSig" | grep '^#'`" ] && ArgToPassVerifyTCSig=""
      SignaturePath=`echo "$EachLine" | awk -F "	" '{print $3}'`
      # If SignaturePath is a comment, ignore it
      [ "`printf "%s" "$SignaturePath" | grep '^#'`" ] && SignaturePath=""
      SecondaryPath=`echo "$EachLine" | awk -F "	" '{print $4}'`
      # If fourth field contains a full file path starting with /, use it as secondary path
      # to determine if PathInFile should be checked
      if [ "`printf "%s" "$SecondaryPath" | grep '^/'`" ] ; then
         # If SecondaryPath does not exist
         if [ ! -e "$SecondaryPath" ] ; then
            continue # Skip check
         fi
      fi
      PathsInFolder=`ShowFullFilePath -a "$Folder$PathInFile"`
      if [ -z "$PathsInFolder" ] ; then
         $InstalledFilesOnly && continue
         PathsInFolder="$Folder$PathInFile"
      fi
      for Path in $PathsInFolder ; do
         # If it's a kext in /Library/Extensions (new for VerifySignedFiles.command 2.0.10)
         if [ "`echo "$PathInFile" | grep '^/Library/Extensions/.*\.kext$'`" ] ; then
            IsMavericksKext=true
         else
            IsMavericksKext=false
         fi
         BypassVerifyTCSigForThisPath=false
         # If FileToUse contains a VerifyTCSigBypassPattern pattern
         if [ -n "$VerifyTCSigBypassPattern" ] ; then
            # If path matches VerifyTCSigBypassPattern
            if [ "`echo "$Path" | egrep -e "$VerifyTCSigBypassPattern"`" ] ; then
               BypassVerifyTCSigForThisPath=true
            fi
         fi
         # If it's a version link that should be resolved
         if [ "`echo "$Path" | grep -e '/Versions/Current/\|/Versions/Current$'`" ] ; then
            TestPath=`ShowFullFilePath -P "$Path"`
            # If physical path exists
            if [ -e "$TestPath" ] ; then
               Path="$TestPath"
            # Else if basename of Path is not "Current"
            elif [ "z`basename "$Path"`" != zCurrent ] ; then
               TestPath=`dirname "$Path"`
               while [ "z`basename "$TestPath"`" != zCurrent ] ; do
                  TestPath=`dirname "$TestPath"`
               done
               TestPath=`dirname "$TestPath"`
               if [ -d "$TestPath" ] ; then
                  ResolvedDirPath=`ls -d "$TestPath/"* | grep -v 'Current$' | tail -n 1`
                  ResolvedDirPathBase=`basename "$ResolvedDirPath"`
                  Path=`echo "$Path" | sed s/"Versions\/Current"/"Versions\/$ResolvedDirPathBase"/`
               fi
            fi
         fi
         if [ ! -e "$Path" ] ; then
            $InstalledFilesOnly && continue
            let MissingFileErrorCount=$MissingFileErrorCount+1
            SomethingShown=true
            echo
            echo "-----------------"
            echo
            echo "NOT FOUND: \"$Path\""
            continue
         fi
         # If no VerifyTCSig argument was specified
         if [ -z "$ArgToPassVerifyTCSig" ] ; then
            # If Path extension is .dylib, set ArgToPassVerifyTCSig to --binary
            [ "`echo "$Path" | grep '\.dylib$'`" ] && ArgToPassVerifyTCSig="--binary"
         # Else if ArgToPassVerifyTCSig is a comment, ignore it
         elif [ "`printf "%s" "$ArgToPassVerifyTCSig" | grep '^#'`" ] ; then
            ArgToPassVerifyTCSig=""
         fi
         if [ "z$SignaturePath" = z--nocodesign ] ; then
            SignaturePath=""
            BypassCodesignCheck=true
         else
            BypassCodesignCheck=false
         fi
         # If NIS 5.x is not installed, do check Symantec signing, do not bypass VerifyTCSig for this path,
         # and an operator was passed
         if [ $NIS5IsInstalled = false -a $CheckSymantecSign = true -a \
              $BypassVerifyTCSigForThisPath = false -a -n "$ArgToPassVerifyTCSig" ] ; then
            # Check for Symantec VerifyTCSig signing
            if [ "z$ArgToPassVerifyTCSig" = z--data ] ; then
               # If signature was not specified in file list
               if [ -z "$SignaturePath" ] ; then
                  # If Path is a plist
                  if [ "`echo "$Path" | grep '\.plist$'`" ] ; then
                     SignaturePath="`echo "$Path" | awk '{print substr($0,1,length($0)-5)}'`sig"
                     [ ! -f "$SignaturePath" ] && SignaturePath="$Path.sig"
                  fi
               fi
            fi
            if [ "$SignaturePath" ] ; then
               "$SymantecVerifyProgram" $ArgToPassVerifyTCSig "$Path" "$SignaturePath" &> "$TempFile"
               ExitCode=$?
            else
               "$SymantecVerifyProgram" $ArgToPassVerifyTCSig "$Path" &> "$TempFile"
               ExitCode=$?
            fi
            if [ $ExitCode != 0 -o $ErrorsOnly = false ] ; then
               [ $ExitCode != 0 ] && let TCSigErrorCount=$TCSigErrorCount+1
               SomethingShown=true
               echo
               echo "-----------------"
               echo
               echo "$SymantecVerifyProgramName $ArgToPassVerifyTCSig \"$Path\"`[ "$SignaturePath" ] && echo " \\"$SignaturePath\\""`"
               echo
               cat "$TempFile"
               $ErrorsOnly || echo "[$SymantecVerifyProgramName returned $ExitCode]"
            fi
         fi
         # If Path is not a text file and do check Apple signing
         if [ "z$ArgToPassVerifyTCSig" != z--data -a $BypassCodesignCheck = false -a $CheckAppleSign = true ] ; then
            # VerifySignedFiles 1.34: ignore codesign validation of SymAPComm.kext if OS is 10.8 or greater
            [ "$PathInFile" = "/Library/Application Support/Symantec/AntiVirus/SymAPComm.kext" -a $macOSVersionNumeric -ge 10008000 ] && continue
            ProgramUsed=$AppleVerifyProgramName
            SigningCallToShow="$AppleVerifyProgramName -vv \"$Path\""
            ExitCode=0
            # If checking NAV libecomlodr.dylib and NIS 6 or later is not installed
            if [ "$PathInFile" = /usr/local/lib/libecomlodr.dylib -a ! -d "$Folder$SiloRootDir" ] ; then
               : # Skip verification
# VerifySignedFiles.command 2.0.47: no longer run kextutil
#             elif $IsMavericksKext ; then
#                # If OS 10.9 or later
#                if [ $macOSVersionNumeric -ge 10009000 ] ; then
#                   # Use kextutil to test kext
#                   ProgramUsed=kextutil
#                   SigningCallToShow="kextutil -nt \"$Path\""
#                   kextutil -nt "$Path" &> "$TempFile"
#                   ExitCode=$?
#                else
#                   continue
#                fi
            else
               # Check for Apple codesign signing
               # If it's a kext in /Library/Extensions, active OS is 10.14 or later, and Silo/NFM/LiveUpdate exists
               if [ $IsMavericksKext = true -a $macOSVersionNumeric -ge 10014000 -a -d "$Folder$SiloRootDir/NFM/LiveUpdate" ] ; then
                  # Also verify that kext is notarized
                  SigningCallToShow="$AppleVerifyProgramName -vv -R='anchor trusted and notarized' \"$Path\""
                  "$AppleVerifyProgramName" -vv -R='anchor trusted and notarized' "$Path" &> "$TempFile"
               else
                  "$AppleVerifyProgramName" -vv "$Path" &> "$TempFile"
               fi
               ExitCode=$?
               if [ $DoReportEnvelopeErrors = false -a -n "`grep -e "$ResourceEnvelopeObsoleteError" "$TempFile"`" ] ; then
                  let NumberOfResourceEnvelopeErrors=$NumberOfResourceEnvelopeErrors+1
                  RemainingError=`grep -ve "$ResourceEnvelopeObsoleteError" "$TempFile"`
                  # If there is an additional error beyond the resource envelope error
                  if [ "$RemainingError" ] ; then
                     # Have that error shown instead of the resource envelope error
                     printf "%s\n" "$RemainingError" > "$TempFile"
                  else
                     # Ignore the resource envelope error
                     ExitCode=0
                  fi
               fi
            fi
            if [ "z$ArgToPassVerifyTCSig" = z--binary ] ; then
               Base=`basename "$Path"`
               # If NIS 5 or later is not installed and it's a framework binary
               if [ $NIS5IsInstalled = false -a -n "`echo "$Path" | grep "/$Base\.framework/.*/$Base$"`" ] ; then
                  StringToShow=""
                  # If there is not an Apple signature
                  if [ "`grep -e "code object is not signed" "$TempFile"`" ] ; then
                     $ErrorsOnly || StringToShow="$AppleVerifyProgramName signature does not exist, which is good since it shouldn't."
                  else
                     StringToShow="ERROR: There is a $AppleVerifyProgramName signature for this file but there shouldn't be."
                  fi
                  if [ "$StringToShow" ] ; then
                     echo
                     echo "-----------------"
                     echo
                     if $ErrorsOnly ; then
                        echo "\"$Path\""
                     else
                        echo "$SigningCallToShow"
                        echo "[$AppleVerifyProgramName returned $ExitCode]"
                     fi
                     echo
                     echo "$StringToShow"
                  fi
                  continue
               fi
            fi
            if [ $ExitCode != 0 -o $ErrorsOnly = false ] ; then
               SomethingShown=true
               echo
               echo "-----------------"
               echo
               echo "$SigningCallToShow"
               echo
               cat "$TempFile"
               $ErrorsOnly || echo "[$ProgramUsed returned $ExitCode]"
               # If codesign did not indicate that the file/folder wasn't signed
               # or that the signature was unrecognized
               if [ -z "`egrep -ie "not signed at all|unrecogniz" "$TempFile"`" ] ; then
                  # Add details about the signature
                  echo
                  echo "$AppleVerifyProgramName -d --verbose=4 \"$Path\""
                  echo
                  "$AppleVerifyProgramName" -d --verbose=4 "$Path"
               fi
               if [ $ExitCode != 0 ] ; then
                  (( AppleSignErrorCount++ ))
                  if $IsMavericksKext ; then
                     echo
                     CodeResourcesFile="$Path/Contents/_CodeSignature/CodeResources"
                     if [ -f "$CodeResourcesFile" ] ; then
                        md5 "$CodeResourcesFile"
                     else
                        echo "NOTE: File does not exist:"
                        echo "      \"$CodeResourcesFile\""
                     fi
                  fi
               fi
            fi
         fi
      done
   done
done
echo
echo "+++++++++++++++++"
echo
let ErrorCount=$AppleSignErrorCount+TCSigErrorCount+$MissingFileErrorCount
if [ $DoReportEnvelopeErrors = false -a $NumberOfResourceEnvelopeErrors -gt 0 ] ; then
   if [ $NumberOfResourceEnvelopeErrors = 1 ] ; then
      echo "There was a \"$ResourceEnvelopeObsoleteError\" error found."
      echo "To see that error in detail, pass -r to $FullScriptName."
   else
      echo "There were $NumberOfResourceEnvelopeErrors \"$ResourceEnvelopeObsoleteError\" errors found."
      echo "To see those errors in detail, pass -r to $FullScriptName."
   fi
fi
if [ $ErrorCount = 0 ] ; then
   if [ $DoReportEnvelopeErrors = false -a $NumberOfResourceEnvelopeErrors -gt 0 ] ; then
      echo "No signature errors were found other than \"$ResourceEnvelopeObsoleteError\"."
   else
      echo "No signature errors were found."
   fi
else
   ScriptExitCode=$ExitCodeWhenSignatureErrorsFound
   [ $AppleSignErrorCount -gt 0 ] && printf "%5s %s\n" "$AppleSignErrorCount" "codesign error`[ $AppleSignErrorCount -gt 1 ] && echo s`"
   [ $TCSigErrorCount -gt 0 ] && printf "%5s %s\n" "$TCSigErrorCount" "VerifyTCSig error`[ $TCSigErrorCount -gt 1 ] && echo s`"
   [ $MissingFileErrorCount -gt 0 ] && printf "%5s %s\n" "$MissingFileErrorCount" "file`[ $MissingFileErrorCount = 1 ] && echo " was" || echo "s were"` not found"
fi
echo
ExitScript $ScriptExitCode

# *** End of Commands to Execute ***
